@@ -3,7 +3,7 @@
 *  C4 dynamic programming library - alignment code               *
 *                                                                *
 *  Guy St.C. Slater..   mailto:guy@ebi.ac.uk                     *
-*  Copyright (C) 2000-2008.  All Rights Reserved.                *
+*  Copyright (C) 2000-2009.  All Rights Reserved.                *
 *                                                                *
 *  This source code is distributed under the terms of the        *
 *  GNU General Public License, version 3. See the file COPYING   *
@@ -76,13 +76,15 @@ void Alignment_display_vulgar(Alignment *alignment,
 
 void Alignment_display_gff(Alignment *alignment,
                            Sequence *query, Sequence *target,
+                           Translate *translate,
                            gboolean report_on_query,
                            gboolean report_on_genomic,
-                           gint result_id, FILE *fp);
+                           gint result_id, gpointer user_data, FILE *fp);
 
 void Alignment_display_ryo(Alignment *alignment,
         Sequence *query, Sequence *target, gchar *format,
-        Translate *translate, gint rank, gpointer user_data, FILE *fp);
+        Translate *translate, gint rank,
+        gpointer user_data, gpointer self_data, FILE *fp);
 
 /**/
 
