@@ -3,7 +3,7 @@
 *  C4 dynamic programming library - code for models              *
 *                                                                *
 *  Guy St.C. Slater..   mailto:guy@ebi.ac.uk                     *
-*  Copyright (C) 2000-2008.  All Rights Reserved.                *
+*  Copyright (C) 2000-2009.  All Rights Reserved.                *
 *                                                                *
 *  This source code is distributed under the terms of the        *
 *  GNU General Public License, version 3. See the file COPYING   *
@@ -520,7 +520,7 @@ C4_Model *C4_Model_create(gchar *name){
     model->exit_func = NULL;
     model->exit_macro = NULL;
     /**/
-    model->ref_count = 1;
+    model->thread_ref = ThreadRef_create();
     model->is_open = TRUE;
     /**/
     model->state_list      = g_ptr_array_new();
@@ -560,7 +560,7 @@ static void C4_Model_destroy_string_list(GPtrArray *string_list){
 void C4_Model_destroy(C4_Model *model){
     register gint i;
     g_assert(model);
-    if(--model->ref_count)
+    if(ThreadRef_destroy(model->thread_ref))
         return;
     g_free(model->name);
     C4_Model_destroy_string_list(model->global_code_list);
@@ -597,7 +597,7 @@ void C4_Model_destroy(C4_Model *model){
 C4_Model *C4_Model_share(C4_Model *model){
     g_assert(model);
     g_assert(!model->is_open);
-    model->ref_count++;
+    ThreadRef_share(model->thread_ref);
     return model;
     }
 
