@@ -3,7 +3,7 @@
 *  Comparison : A module for pairwise sequence comparisons       *
 *                                                                *
 *  Guy St.C. Slater..   mailto:guy@ebi.ac.uk                     *
-*  Copyright (C) 2000-2008.  All Rights Reserved.                *
+*  Copyright (C) 2000-2009.  All Rights Reserved.                *
 *                                                                *
 *  This source code is distributed under the terms of the        *
 *  GNU General Public License, version 3. See the file COPYING   *
@@ -36,7 +36,7 @@ static Comparison_Param *Comparison_Param_create_without_mirror(
                                           HSP_Param *codon_hsp_param,
                                           gboolean swap_param){
     register Comparison_Param *param = g_new(Comparison_Param, 1);
-    param->ref_count = 1;
+    param->thread_ref = NULL;
     param->query_type = query_type;
     param->target_type = target_type;
     param->dna_hsp_param = Comparison_Param_get_hsp_param(dna_hsp_param,
@@ -64,15 +64,10 @@ Comparison_Param *Comparison_Param_create(Alphabet_Type query_type,
                             dna_hsp_param, protein_hsp_param,
                             codon_hsp_param, TRUE);
     param->mirror->mirror = param;
+    param->mirror->thread_ref = param->thread_ref = ThreadRef_create();
     return param;
     }
 
-static gint Comparison_Param_ref_count(Comparison_Param *param){
-    g_assert(param);
-    g_assert(param->mirror);
-    return param->ref_count + param->mirror->ref_count;
-    }
-
 static void Comparison_Param_destroy_without_mirror(
             Comparison_Param *param){
     g_assert(param);
@@ -89,8 +84,7 @@ static void Comparison_Param_destroy_without_mirror(
 
 void Comparison_Param_destroy(Comparison_Param *param){
     g_assert(param);
-    param->ref_count--;
-    if(Comparison_Param_ref_count(param) > 1) /* Allow for mirror */
+    if(ThreadRef_destroy(param->thread_ref))
         return;
     Comparison_Param_destroy_without_mirror(param->mirror);
     Comparison_Param_destroy_without_mirror(param);
@@ -99,7 +93,7 @@ void Comparison_Param_destroy(Comparison_Param *param){
 
 Comparison_Param *Comparison_Param_share(Comparison_Param *param){
     g_assert(param);
-    param->ref_count++;
+    ThreadRef_share(param->thread_ref);
     return param;
     }
 
@@ -132,11 +126,12 @@ Comparison *Comparison_create(Comparison_Param *param,
     g_assert(param);
     g_assert(query);
     g_assert(target);
-    comparison->ref_count = 1;
+    comparison->thread_ref = ThreadRef_create();
     comparison->param = Comparison_Param_share(param);
     comparison->query = Sequence_share(query);
     comparison->target = Sequence_share(target);
-    comparison->dna_hspset = param->dna_hsp_param
+    ThreadRef_lock(comparison->param->thread_ref);
+    comparison->dna_hspset = comparison->param->dna_hsp_param
                            ? HSPset_create(query, target,
                                            param->dna_hsp_param)
                            : NULL;
@@ -148,17 +143,18 @@ Comparison *Comparison_create(Comparison_Param *param,
                              ? HSPset_create(query, target,
                                              param->codon_hsp_param)
                              : NULL;
+    ThreadRef_unlock(comparison->param->thread_ref);
     return comparison;
     }
 
 Comparison *Comparison_share(Comparison *comparison){
-    comparison->ref_count++;
+    ThreadRef_share(comparison->thread_ref);
     return comparison;
     }
 
 void Comparison_destroy(Comparison *comparison){
     g_assert(comparison);
-    if(--comparison->ref_count)
+    if(ThreadRef_destroy(comparison->thread_ref))
         return;
     Comparison_Param_destroy(comparison->param);
     Sequence_destroy(comparison->query);
@@ -217,7 +213,7 @@ void Comparison_finalise(Comparison *comparison){
 
 void Comparison_swap(Comparison *comparison){
     register Sequence *query;
-    g_assert(comparison->ref_count == 1);
+    g_assert(ThreadRef_get_count(comparison->thread_ref) == 1);
     /* Mirror parameters */
     comparison->param = Comparison_Param_share(comparison->param->mirror);
     Comparison_Param_destroy(comparison->param->mirror);
@@ -242,7 +238,7 @@ void Comparison_swap(Comparison *comparison){
 void Comparison_revcomp(Comparison *comparison){
     register Sequence *rc_query = Sequence_revcomp(comparison->query),
                       *rc_target = Sequence_revcomp(comparison->target);
-    g_assert(comparison->ref_count == 1);
+    g_assert(ThreadRef_get_count(comparison->thread_ref) == 1);
     g_assert(comparison->dna_hspset);
     g_assert(!comparison->protein_hspset);
     g_assert(!comparison->codon_hspset);
