@@ -3,7 +3,7 @@
 *  Comparison : A module for pairwise sequence comparisons       *
 *                                                                *
 *  Guy St.C. Slater..   mailto:guy@ebi.ac.uk                     *
-*  Copyright (C) 2000-2008.  All Rights Reserved.                *
+*  Copyright (C) 2000-2009.  All Rights Reserved.                *
 *                                                                *
 *  This source code is distributed under the terms of the        *
 *  GNU General Public License, version 3. See the file COPYING   *
@@ -27,9 +27,10 @@ extern "C" {
 #include "translate.h"
 #include "submat.h"
 #include "hspset.h"
+#include "threadref.h"
 
 typedef struct Comparison_Param {
-                      gint  ref_count;
+                 ThreadRef *thread_ref;
              Alphabet_Type  query_type;
              Alphabet_Type  target_type;
                  HSP_Param *dna_hsp_param;
@@ -50,7 +51,7 @@ Comparison_Param *Comparison_Param_swap(Comparison_Param *param);
 /**/
 
 typedef struct {
-                gint  ref_count;
+           ThreadRef *thread_ref;
     Comparison_Param *param;
             Sequence *query;
             Sequence *target;
@@ -64,7 +65,8 @@ Comparison *Comparison_create(Comparison_Param *param,
 Comparison *Comparison_share(Comparison *comparison);
 void Comparison_destroy(Comparison *comparison);
 void Comparison_print(Comparison *comparison);
-HSPset_ArgumentSet *Comparison_Param_get_HSPSet_Argument_Set(Comparison_Param *param);
+HSPset_ArgumentSet *Comparison_Param_get_HSPSet_Argument_Set(
+                    Comparison_Param *param);
 
 gboolean  Comparison_has_hsps(Comparison *comparison);
     void  Comparison_finalise(Comparison *comparison);
