@@ -3,7 +3,7 @@
 *  Library for HSP sets (high-scoring segment pairs)             *
 *                                                                *
 *  Guy St.C. Slater..   mailto:guy@ebi.ac.uk                     *
-*  Copyright (C) 2000-2008.  All Rights Reserved.                *
+*  Copyright (C) 2000-2009.  All Rights Reserved.                *
 *                                                                *
 *  This source code is distributed under the terms of the        *
 *  GNU General Public License, version 3. See the file COPYING   *
@@ -66,13 +66,13 @@ HSPset_ArgumentSet *HSPset_ArgumentSet_create(Argument *arg){
         /**/
         ArgumentSet_add_option(as, 0, "dnawordlimit", "score",
             "Score limit for dna word neighbourhood", "0",
-            Argument_parse_int, &has.dna_wordlimit);
+            Argument_parse_int, &has.dna_word_limit);
         ArgumentSet_add_option(as, 0, "proteinwordlimit", "score",
             "Score limit for protein word neighbourhood", "4",
-            Argument_parse_int, &has.protein_wordlimit);
+            Argument_parse_int, &has.protein_word_limit);
         ArgumentSet_add_option(as, 0, "codonwordlimit", "score",
             "Score limit for codon word neighbourhood", "4",
-            Argument_parse_int, &has.codon_wordlimit);
+            Argument_parse_int, &has.codon_word_limit);
         /**/
         ArgumentSet_add_option(as, '\0', "geneseed", "threshold",
              "Geneseed Threshold", "0",
@@ -142,6 +142,61 @@ void HSP_Param_set_codon_hsp_threshold(HSP_Param *hsp_param,
 
 /**/
 
+static void HSP_Param_refresh_wordhood(HSP_Param *hsp_param){
+    register WordHood_Alphabet *wha = NULL;
+    register Submat *submat;
+    if(hsp_param->wordhood)
+        WordHood_destroy(hsp_param->wordhood);
+    if(hsp_param->has->use_wordhood_dropoff && (!hsp_param->wordlimit)){
+        hsp_param->wordhood = NULL;
+    } else {
+        submat = (hsp_param->match->type == Match_Type_DNA2DNA)
+               ? hsp_param->match->mas->dna_submat
+               : hsp_param->match->mas->protein_submat;
+        wha = WordHood_Alphabet_create_from_Submat(
+                (gchar*)hsp_param->match->comparison_alphabet->member,
+                (gchar*)hsp_param->match->comparison_alphabet->member,
+                submat, FALSE);
+        g_assert(wha);
+        hsp_param->wordhood = WordHood_create(wha, hsp_param->wordlimit,
+                                       hsp_param->has->use_wordhood_dropoff);
+        WordHood_Alphabet_destroy(wha);
+        }
+    return;
+    }
+/* FIXME: optimisation: do not free/recreate wordhood when unnecessary */
+
+void HSP_Param_set_dna_word_limit(HSP_Param *hsp_param,
+                                  gint dna_word_limit){
+    if(hsp_param->match->type == Match_Type_DNA2DNA){
+        hsp_param->wordlimit = dna_word_limit;
+        HSP_Param_refresh_wordhood(hsp_param);
+        }
+    return;
+    }
+
+void HSP_Param_set_protein_word_limit(HSP_Param *hsp_param,
+                                      gint protein_word_limit){
+    if((hsp_param->match->type == Match_Type_PROTEIN2PROTEIN)
+    || (hsp_param->match->type == Match_Type_PROTEIN2DNA)
+    || (hsp_param->match->type == Match_Type_DNA2PROTEIN)){
+        hsp_param->wordlimit = protein_word_limit;
+        HSP_Param_refresh_wordhood(hsp_param);
+        }
+    return;
+    }
+
+void HSP_Param_set_codon_word_limit(HSP_Param *hsp_param,
+                                    gint codon_word_limit){
+    if(hsp_param->match->type == Match_Type_CODON2CODON){
+        hsp_param->threshold = codon_word_limit;
+        HSP_Param_refresh_wordhood(hsp_param);
+        }
+    return;
+    }
+
+/**/
+
 void HSP_Param_set_dna_hsp_dropoff(HSP_Param *hsp_param,
                                    gint dna_hsp_dropoff){
     if(hsp_param->match->type == Match_Type_DNA2DNA)
@@ -181,9 +236,7 @@ void HSP_Param_set_seed_repeat(HSP_Param *hsp_param,
 
 HSP_Param *HSP_Param_create(Match *match, gboolean use_horizon){
     register HSP_Param *hsp_param = g_new(HSP_Param, 1);
-    register WordHood_Alphabet *wha = NULL;
-    register Submat *submat;
-    hsp_param->ref_count = 1;
+    hsp_param->thread_ref = ThreadRef_create();
     hsp_param->has = HSPset_ArgumentSet_create(NULL);
     hsp_param->match = match;
     hsp_param->seed_repeat = hsp_param->has->seed_repeat;
@@ -192,7 +245,7 @@ HSP_Param *HSP_Param_create(Match *match, gboolean use_horizon){
             hsp_param->dropoff = hsp_param->has->dna_hsp_dropoff;
             hsp_param->threshold = hsp_param->has->dna_hsp_threshold;
             HSP_Param_set_wordlen(hsp_param, hsp_param->has->dna_wordlen);
-            hsp_param->wordlimit = hsp_param->has->dna_wordlimit;
+            hsp_param->wordlimit = hsp_param->has->dna_word_limit;
             break;
         case Match_Type_PROTEIN2PROTEIN: /*fallthrough*/
         case Match_Type_PROTEIN2DNA:     /*fallthrough*/
@@ -201,13 +254,13 @@ HSP_Param *HSP_Param_create(Match *match, gboolean use_horizon){
             hsp_param->threshold
                 = hsp_param->has->protein_hsp_threshold;
             HSP_Param_set_wordlen(hsp_param, hsp_param->has->protein_wordlen);
-            hsp_param->wordlimit = hsp_param->has->protein_wordlimit;
+            hsp_param->wordlimit = hsp_param->has->protein_word_limit;
             break;
         case Match_Type_CODON2CODON:
             hsp_param->dropoff = hsp_param->has->codon_hsp_dropoff;
             hsp_param->threshold = hsp_param->has->codon_hsp_threshold;
             HSP_Param_set_wordlen(hsp_param, hsp_param->has->codon_wordlen);
-            hsp_param->wordlimit = hsp_param->has->codon_wordlimit;
+            hsp_param->wordlimit = hsp_param->has->codon_word_limit;
             g_assert(!(hsp_param->wordlen % 3));
             break;
         default:
@@ -215,36 +268,36 @@ HSP_Param *HSP_Param_create(Match *match, gboolean use_horizon){
             break;
         }
     hsp_param->use_horizon = use_horizon;
-    if(hsp_param->has->use_wordhood_dropoff && (!hsp_param->wordlimit)){
-        hsp_param->wordhood = NULL;
-    } else {
-        submat = (match->type == Match_Type_DNA2DNA)
-               ? match->mas->dna_submat
-               : match->mas->protein_submat;
-        wha = WordHood_Alphabet_create_from_Submat(
-                (gchar*)match->comparison_alphabet->member,
-                (gchar*)match->comparison_alphabet->member, submat, FALSE);
-        g_assert(wha);
-        hsp_param->wordhood = WordHood_create(wha, hsp_param->wordlimit,
-                                          hsp_param->has->use_wordhood_dropoff);
-        WordHood_Alphabet_destroy(wha);
-        }
+#ifdef USE_PTHREADS
+    pthread_mutex_init(&hsp_param->hsp_recycle_lock, NULL);
+#endif /* USE_PTHREADS */
+    hsp_param->hsp_recycle = RecycleBin_create("HSP", sizeof(HSP), 64);
+    hsp_param->wordhood = NULL;
+    HSP_Param_refresh_wordhood(hsp_param);
     return hsp_param;
     }
 
 void HSP_Param_destroy(HSP_Param *hsp_param){
     g_assert(hsp_param);
-    if(--hsp_param->ref_count)
+    if(ThreadRef_destroy(hsp_param->thread_ref))
         return;
     if(hsp_param->wordhood)
         WordHood_destroy(hsp_param->wordhood);
+#ifdef USE_PTHREADS
+    pthread_mutex_lock(&hsp_param->hsp_recycle_lock);
+#endif /* USE_PTHREADS */
+    RecycleBin_destroy(hsp_param->hsp_recycle);
+#ifdef USE_PTHREADS
+    pthread_mutex_unlock(&hsp_param->hsp_recycle_lock);
+    pthread_mutex_destroy(&hsp_param->hsp_recycle_lock);
+#endif /* USE_PTHREADS */
     g_free(hsp_param);
     return;
     }
 
 HSP_Param *HSP_Param_share(HSP_Param *hsp_param){
     g_assert(hsp_param);
-    hsp_param->ref_count++;
+    ThreadRef_share(hsp_param->thread_ref);
     return hsp_param;
     }
 
@@ -254,8 +307,6 @@ HSP_Param *HSP_Param_swap(HSP_Param *hsp_param){
                             hsp_param->use_horizon);
     }
 
-static RecycleBin *global_hsp_recycle_bin = NULL;
-
 HSPset *HSPset_create(Sequence *query, Sequence *target,
                       HSP_Param *hsp_param){
     register HSPset *hsp_set = g_new(HSPset, 1);
@@ -266,15 +317,9 @@ HSPset *HSPset_create(Sequence *query, Sequence *target,
     hsp_set->target = Sequence_share(target);
     hsp_set->param = HSP_Param_share(hsp_param);
     /**/
-    if(global_hsp_recycle_bin){
-        hsp_set->hsp_recycle = RecycleBin_share(global_hsp_recycle_bin);
-    } else {
-        global_hsp_recycle_bin = RecycleBin_create("HSP", sizeof(HSP), 64);
-        hsp_set->hsp_recycle = global_hsp_recycle_bin;
-        }
     if(hsp_param->use_horizon){
         hsp_set->horizon = (gint****)Matrix4d_create(
-                                   1 + ((hsp_param->seed_repeat > 1)?1:0),
+                                   1 + ((hsp_param->seed_repeat > 1)?2:0),
                                    query->len,
                                    hsp_param->match->query->advance,
                                    hsp_param->match->target->advance,
@@ -295,6 +340,11 @@ HSPset *HSPset_create(Sequence *query, Sequence *target,
     hsp_set->is_empty = TRUE;
     return hsp_set;
     }
+/* horizon[score(,repeat_count,diag)]
+ *        [query_len]
+ *        [query_advance]
+ *        [target_advance]
+ */
 
 /**/
 
@@ -322,9 +372,6 @@ void HSPset_destroy(HSPset *hsp_set){
         HSP_destroy(hsp);
         }
     g_ptr_array_free(hsp_set->hsp_list, TRUE);
-    if(hsp_set->hsp_recycle->ref_count == 1) /* last active hsp_set */
-        global_hsp_recycle_bin = NULL;
-    RecycleBin_destroy(hsp_set->hsp_recycle);
     if(hsp_set->horizon)
         g_free(hsp_set->horizon);
     g_free(hsp_set);
@@ -520,9 +567,9 @@ static void HSP_Display_print(HSP_Display *hd){
         }
     HSP_Display_print_ruler(hd, hd->top->len-pos, pos, TRUE);
     g_print(" query:[%.*s]\n       [%.*s]\ntarget:[%.*s]\n",
-            hd->top->len-pos, hd->top->str+pos,
-            hd->mid->len-pos, hd->mid->str+pos,
-            hd->low->len-pos, hd->low->str+pos);
+            (gint)(hd->top->len-pos), hd->top->str+pos,
+            (gint)(hd->mid->len-pos), hd->mid->str+pos,
+            (gint)(hd->low->len-pos), hd->low->str+pos);
     HSP_Display_print_ruler(hd, hd->top->len-pos, pos, FALSE);
     g_print("\n");
     return;
@@ -766,11 +813,18 @@ static void HSP_extend(HSP *nh, gboolean forbid_masked){
     return;
     }
 /* The score cannot be allowed to drop below zero in the HSP,
- * as this can result in overlapping HSPs in some circurmstances.
+ * as this can result in overlapping HSPs in some circumstances.
  */
 
 static HSP *HSP_create(HSP *nh){
-    register HSP *hsp = RecycleBin_alloc(nh->hsp_set->hsp_recycle);
+    register HSP *hsp;
+#ifdef USE_PTHREADS
+    pthread_mutex_lock(&nh->hsp_set->param->hsp_recycle_lock);
+#endif /* USE_PTHREADS */
+    hsp = RecycleBin_alloc(nh->hsp_set->param->hsp_recycle);
+#ifdef USE_PTHREADS
+    pthread_mutex_unlock(&nh->hsp_set->param->hsp_recycle_lock);
+#endif /* USE_PTHREADS */
     hsp->hsp_set = nh->hsp_set;
     hsp->query_start = nh->query_start;
     hsp->target_start = nh->target_start;
@@ -782,7 +836,13 @@ static HSP *HSP_create(HSP *nh){
 
 void HSP_destroy(HSP *hsp){
     register HSPset *hsp_set = hsp->hsp_set;
-    RecycleBin_recycle(hsp_set->hsp_recycle, hsp);
+#ifdef USE_PTHREADS
+    pthread_mutex_lock(&hsp_set->param->hsp_recycle_lock);
+#endif /* USE_PTHREADS */
+    RecycleBin_recycle(hsp_set->param->hsp_recycle, hsp);
+#ifdef USE_PTHREADS
+    pthread_mutex_unlock(&hsp_set->param->hsp_recycle_lock);
+#endif /* USE_PTHREADS */
     return;
     }
 
@@ -885,6 +945,16 @@ void HSPset_seed_hsp(HSPset *hsp_set,
     /**/
     g_assert(section_pos >= 0);
     g_assert(section_pos < hsp_set->query->len);
+    /* Clear if diag has changed */
+    if(hsp_set->param->seed_repeat > 1){
+        if(hsp_set->horizon[2][section_pos][query_frame][target_frame]
+               != (diag_pos + hsp_set->query->len)){
+            hsp_set->horizon[0][section_pos][query_frame][target_frame] = 0;
+            hsp_set->horizon[1][section_pos][query_frame][target_frame] = 0;
+            hsp_set->horizon[2][section_pos][query_frame][target_frame]
+                = diag_pos + hsp_set->query->len;
+            }
+        }
     /* Check whether we have seen this HSP already */
     if(target_start < hsp_set->horizon[0]
                                       [section_pos]
@@ -963,6 +1033,7 @@ static void HSPset_seed_hsp_sorted(HSPset *hsp_set,
         horizon[0][query_frame][target_frame] = 0;
         horizon[2][query_frame][target_frame] = 0;
         }
+/* FIXME: seedrepeat overflow here */
     if(++horizon[2][query_frame][target_frame] < hsp_set->param->seed_repeat)
         return;
     horizon[2][query_frame][target_frame] = 0;
