@@ -3,7 +3,7 @@
 *  Seeder : A module for seeding pairwise alignments             *
 *                                                                *
 *  Guy St.C. Slater..   mailto:guy@ebi.ac.uk                     *
-*  Copyright (C) 2000-2008.  All Rights Reserved.                *
+*  Copyright (C) 2000-2009.  All Rights Reserved.                *
 *                                                                *
 *  This source code is distributed under the terms of the        *
 *  GNU General Public License, version 3. See the file COPYING   *
@@ -25,13 +25,17 @@
      (*(type*)((char*)(instance) + (offset)))
 #endif /* OFFSET_ITEM */
 
+#ifndef Swap
+#define Swap(x,y,temp) ((temp)=(x),(x)=(y),(y)=(temp))
+#endif /* Swap */
+
 Seeder_ArgumentSet *Seeder_ArgumentSet_create(Argument *arg){
     register ArgumentSet *as;
     static Seeder_ArgumentSet sas;
     if(arg){
         as = ArgumentSet_create("Alignment Seeding Options");
         ArgumentSet_add_option(as, 'M', "fsmmemory", "Mb",
-                "Memory limit for FSM scanning", "64",
+                "Memory limit for FSM scanning", "256",
                 Argument_parse_int, &sas.fsm_memory_limit);
         ArgumentSet_add_option(as, '\0', "forcefsm", "fsm type",
             "Force FSM type ( normal | compact )", "none",
@@ -39,6 +43,9 @@ Seeder_ArgumentSet *Seeder_ArgumentSet_create(Argument *arg){
         ArgumentSet_add_option(as, 0, "wordjump", NULL,
             "Jump between query words", "1",
             Argument_parse_int, &sas.word_jump);
+        ArgumentSet_add_option(as, 0, "wordambiguity", NULL,
+            "Number of ambiguous words to expand", "1",
+            Argument_parse_int, &sas.word_ambiguity);
         Argument_absorb_ArgumentSet(arg, as);
         }
     return &sas;
@@ -302,6 +309,14 @@ Seeder *Seeder_create(gint verbosity,
                       gpointer user_data){
     register Seeder *seeder = g_new0(Seeder, 1);
     seeder->sas = Seeder_ArgumentSet_create(NULL);
+    if(seeder->sas->word_ambiguity < 1)
+        g_error("Word ambiguity cannot be less than one.");
+    if(seeder->sas->word_ambiguity > 1){
+        if(comparison_param->protein_hsp_param)
+            g_error("Protein ambuiguity symbols not implemented");
+        g_warning("setting --wordambiguity in exonerate will be SLOW.");
+        g_warning("... use esd2esi --wordambiguity and exonerate-server instead");
+        }
     seeder->is_prepared = FALSE;
     seeder->verbosity = verbosity;
     seeder->report_func = report_func;
@@ -413,25 +428,25 @@ gsize Seeder_memory_usage(Seeder *seeder){
     }
 
 void Seeder_memory_info(Seeder *seeder){
-    g_message("Seeder Total = %dMb", Seeder_memory_usage(seeder)>>20);
+    g_message("Seeder Total = %dMb", (gint)(Seeder_memory_usage(seeder)>>20));
     g_assert(seeder);
     if(seeder->seeder_fsm)
         g_message(" -> FSM memory = %dMb",
-                Seeder_FSM_memory_usage(seeder->seeder_fsm)>>20);
+                (gint)(Seeder_FSM_memory_usage(seeder->seeder_fsm)>>20));
     else
         g_message(" -> VFSM memory = %dMb",
-                Seeder_VFSM_memory_usage(seeder->seeder_vfsm)>>20);
+                (gint)(Seeder_VFSM_memory_usage(seeder->seeder_vfsm)>>20));
     g_message(" -> QueryInfo memory = %dMb",
-           (seeder->query_info_list->len
+           (gint)(seeder->query_info_list->len
             *(sizeof(gpointer)+sizeof(Seeder_QueryInfo)))>>20);
     g_message(" -> Recycle_Wordinfo Memory = %dMb",
-           RecycleBin_memory_usage(seeder->recycle_wordinfo)>>20);
+           (gint)(RecycleBin_memory_usage(seeder->recycle_wordinfo)>>20));
     g_message(" -> Recycle_Seed Memory = %dMb",
-           RecycleBin_memory_usage(seeder->recycle_seed)>>20);
+           (gint)(RecycleBin_memory_usage(seeder->recycle_seed)>>20));
     g_message(" -> Recycle_Neighbour Memory = %dMb",
-           RecycleBin_memory_usage(seeder->recycle_neighbour)>>20);
+           (gint)(RecycleBin_memory_usage(seeder->recycle_neighbour)>>20));
     g_message(" -> Recycle_Context Memory = %dMb",
-           RecycleBin_memory_usage(seeder->recycle_context)>>20);
+           (gint)(RecycleBin_memory_usage(seeder->recycle_context)>>20));
     return;
     }
 
@@ -515,8 +530,10 @@ static void Seeder_insert_query(Seeder *seeder, Seeder_Context *context,
         } else {
             leaf = VFSM_state2leaf(vfsm, state);
             word_info = seeder->seeder_vfsm->leaf[leaf];
-            if(!word_info)
+            if(!word_info){
                 word_info = Seeder_WordInfo_create(seeder);
+                seeder->seeder_vfsm->leaf[leaf] = word_info;
+                }
             }
         g_assert(word_info);
         if(frame)
@@ -675,8 +692,8 @@ static void Seeder_FSM_traverse_func(guint seq_pos,
     return;
     }
 
-static void Seeder_VFSM_traverse(Seeder *seeder, gchar *seq,
-                                 Seeder_TargetInfo *target_info){
+static void Seeder_VFSM_traverse_single(Seeder *seeder, gchar *seq,
+                                        Seeder_TargetInfo *target_info){
     register gint i, ch;
     register VFSM_Int state = 0, leaf;
     register VFSM *vfsm = seeder->seeder_vfsm->vfsm;
@@ -700,6 +717,68 @@ static void Seeder_VFSM_traverse(Seeder *seeder, gchar *seq,
     return;
     }
 
+static void Seeder_VFSM_traverse_ambig(Seeder *seeder, gchar *seq,
+                                       Seeder_TargetInfo *target_info){
+    register gint i, j, k, ch;
+    register VFSM_Int state, next_state, leaf;
+    register VFSM *vfsm = seeder->seeder_vfsm->vfsm;
+    register Seeder_WordInfo *word_info;
+    register gchar *ambig;
+    register VFSM_Int *temp_state_list,
+                      *curr_state_list = g_new0(VFSM_Int,
+                                                seeder->sas->word_ambiguity),
+                      *next_state_list = g_new0(VFSM_Int,
+                                                seeder->sas->word_ambiguity);
+    register gint curr_state_list_len = 1, next_state_list_len = 0;
+    g_assert(vfsm);
+    for(i = 0; seq[i]; i++){
+        ch = toupper(seq[i]); /* FIXME: filter properly */
+        if((!(ambig = Alphabet_nt2ambig(ch)))
+        || ((strlen(ambig) * curr_state_list_len)
+        > seeder->sas->word_ambiguity)){
+            next_state_list_len = 0;
+            curr_state_list_len = 1;
+            curr_state_list[0] = 0;
+            continue;
+            }
+        for(j = 0; j < curr_state_list_len; j++){
+            state = curr_state_list[j];
+            for(k = 0; ambig[k]; k++){
+                g_assert(vfsm->index[(guchar)ambig[k]]);
+                next_state = VFSM_change_state_M(vfsm,
+                                  state, (guchar)ambig[k]);
+                /* FIXME: use POW2 versions where applicable */
+                if(next_state_list_len
+                && ((next_state == next_state_list[0])
+                 || (next_state == next_state_list[next_state_list_len-1])))
+                    break;
+                next_state_list[next_state_list_len++] = next_state;
+                if(VFSM_state_is_leaf(vfsm, next_state)){
+                    leaf = VFSM_state2leaf(vfsm, next_state);
+                    word_info = seeder->seeder_vfsm->leaf[leaf];
+                    if(word_info)
+                        Seeder_FSM_traverse_func(i, word_info, target_info);
+                    }
+                }
+            }
+        curr_state_list_len = next_state_list_len;
+        next_state_list_len = 0;
+        Swap(next_state_list, curr_state_list, temp_state_list);
+        }
+    g_free(curr_state_list);
+    g_free(next_state_list);
+    return;
+    }
+
+static void Seeder_VFSM_traverse(Seeder *seeder, gchar *seq,
+                                 Seeder_TargetInfo *target_info){
+    if(seeder->sas->word_ambiguity > 1)
+        Seeder_VFSM_traverse_ambig(seeder, seq, target_info);
+    else
+        Seeder_VFSM_traverse_single(seeder, seq, target_info);
+    return;
+    }
+
 static void Seeder_prepare(Seeder *seeder){
     g_assert(!seeder->is_prepared);
     if(seeder->seeder_fsm)
@@ -708,6 +787,67 @@ static void Seeder_prepare(Seeder *seeder){
     return;
     }
 
+static void Seeder_FSM_traverse_ambig(Seeder *seeder, gchar *seq,
+                                      FSM_Traverse_Func ftf, gpointer user_data){
+    register FSM *f = seeder->seeder_fsm->fsm;
+    register FSM_Node *n, *next_state;
+    register gint c;
+    register gchar *ambig;
+    register gint i, j, k;
+    register FSM_Node **temp_state_list,
+                      **curr_state_list = g_new0(FSM_Node*,
+                                                seeder->sas->word_ambiguity),
+                      **next_state_list = g_new0(FSM_Node*,
+                                                seeder->sas->word_ambiguity);
+    register gint curr_state_list_len = 1, next_state_list_len = 0;
+    g_assert(f->is_compiled);
+    curr_state_list[0] = f->root;
+    for(i = 0; seq[i]; i++){
+        if((!(ambig = Alphabet_nt2ambig(seq[i])))
+        || ((strlen(ambig) * curr_state_list_len)
+            > seeder->sas->word_ambiguity)){
+            next_state_list_len = 0;
+            curr_state_list_len = 1;
+            curr_state_list[0] = f->root;
+            continue;
+            }
+        for(j = 0; j < curr_state_list_len; j++){
+            n = curr_state_list[j];
+            for(k = 0; ambig[k]; k++){
+                c = f->traversal_filter[(guchar)ambig[k]];
+                if(n[c].data)
+                    ftf(i, n[c].data, user_data);
+                next_state = n[c].next;
+                if(next_state_list_len
+                && ((next_state == next_state_list[0])
+                   || (next_state ==
+                       next_state_list[next_state_list_len-1]))){
+                    break;
+                    }
+                next_state_list[next_state_list_len++] = next_state;
+                }
+            }
+        curr_state_list_len = next_state_list_len;
+        next_state_list_len = 0;
+        /* g_message("set curr [%d]", curr_state_list_len); */
+        Swap(next_state_list, curr_state_list, temp_state_list);
+        }
+    g_free(curr_state_list);
+    g_free(next_state_list);
+    return;
+    }
+/* Slow (ambiguous) version of FSM_traverse */
+/* FIXME: optimisation: remove strlen() etc */
+
+static void Seeder_FSM_traverse(Seeder *seeder, gchar *seq, FSM_Traverse_Func ftf,
+                                gpointer user_data){
+    if(seeder->sas->word_ambiguity > 1)
+        Seeder_FSM_traverse_ambig(seeder, seq, ftf, user_data);
+    else
+        FSM_traverse(seeder->seeder_fsm->fsm, seq, ftf, user_data);
+    return;
+    }
+
 void Seeder_add_target(Seeder *seeder, Sequence *target){
     register gint i;
     register Sequence *aa_seq;
@@ -735,8 +875,8 @@ void Seeder_add_target(Seeder *seeder, Sequence *target){
             seq = Sequence_get_str(target_masked);
             Sequence_destroy(target_masked);
             if(seeder->seeder_fsm)
-                FSM_traverse(seeder->seeder_fsm->fsm, seq,
-                             Seeder_FSM_traverse_func, &target_info);
+                Seeder_FSM_traverse(seeder, seq,
+                                    Seeder_FSM_traverse_func, &target_info);
             else
                 Seeder_VFSM_traverse(seeder, seq, &target_info);
             g_free(seq);
@@ -747,8 +887,8 @@ void Seeder_add_target(Seeder *seeder, Sequence *target){
         seq = Sequence_get_str(target_masked);
         Sequence_destroy(target_masked);
         if(seeder->seeder_fsm){
-            FSM_traverse(seeder->seeder_fsm->fsm, seq,
-                    Seeder_FSM_traverse_func, &target_info);
+            Seeder_FSM_traverse(seeder, seq,
+                                Seeder_FSM_traverse_func, &target_info);
         } else {
             Seeder_VFSM_traverse(seeder, seq, &target_info);
             }
