@@ -3,7 +3,7 @@
 *  Library for manipulation of exonerate dataset files           *
 *                                                                *
 *  Guy St.C. Slater..   mailto:guy@ebi.ac.uk                     *
-*  Copyright (C) 2000-2008.  All Rights Reserved.                *
+*  Copyright (C) 2000-2009.  All Rights Reserved.                *
 *                                                                *
 *  This source code is distributed under the terms of the        *
 *  GNU General Public License, version 3. See the file COPYING   *
@@ -23,7 +23,7 @@
 /**/
 
 #define DATASET_HEADER_MAGIC (('e' << 16)|('s' << 8)|('d'))
-#define DATASET_HEADER_VERSION 2
+#define DATASET_HEADER_VERSION 3
 
 static Dataset_Header *Dataset_Header_create(Alphabet_Type alphabet_type,
                                              gboolean softmask_input){
@@ -301,6 +301,23 @@ gsize Dataset_memory_usage(Dataset *dataset){
          + dataset_sequence_memory;
     }
 
+void Dataset_info(Dataset *dataset){
+    g_print("Sequence Dataset:\n"
+            "----------------\n"
+            "             version: %d\n"
+            "                type: %s, %s\n"
+            "          total size: %d Mb\n"
+            " number of sequences: %lld\n"
+            "    longest sequence: %lld\n\n",
+            (gint)dataset->header->version,
+            (dataset->header->type & 1)?"DNA":"PROTEIN",
+            (dataset->header->type & 2)?"masked":"unmasked",
+            (gint)(dataset->header->total_db_len >> 20),
+            dataset->header->number_of_seqs,
+            dataset->header->max_seq_len);
+    return;
+    }
+
 void Dataset_destroy(Dataset *dataset){
     register gint i;
     register Dataset_Sequence *seq;
@@ -378,10 +395,10 @@ static void Dataset_write_seq_data(Dataset *dataset, FILE *fp){
     }
 
 static void Dataset_read_seq_data(Dataset *dataset, FILE *fp){
-    register gint i, j = 0, ipos, dpos;
+    register gint64 i, j = 0, ipos, dpos;
     register Dataset_Sequence *ds;
-    register gint len = dataset->header->seq_info_offset
-                      - dataset->header->seq_data_offset;
+    register guint64 len = dataset->header->seq_info_offset
+                         - dataset->header->seq_data_offset;
     register gchar *buf = g_new(gchar, len+1);
     if(!fread(buf, sizeof(gchar), len, fp))
         g_error("Problem reading seq data");
