@@ -3,7 +3,7 @@
 *  Library for manipulation of FASTA format databases            *
 *                                                                *
 *  Guy St.C. Slater..   mailto:guy@ebi.ac.uk                     *
-*  Copyright (C) 2000-2008.  All Rights Reserved.                *
+*  Copyright (C) 2000-2009.  All Rights Reserved.                *
 *                                                                *
 *  This source code is distributed under the terms of the        *
 *  GNU General Public License, version 3. See the file COPYING   *
@@ -202,6 +202,19 @@ FastaDB *FastaDB_share(FastaDB *fdb){
     return fdb;
     }
 
+FastaDB *FastaDB_dup(FastaDB *fdb){
+    register FastaDB *nfdb = g_new(FastaDB, 1);
+    nfdb->ref_count = 1;
+    nfdb->alphabet = Alphabet_create(fdb->alphabet->type,
+                                     fdb->alphabet->is_soft_masked);
+    nfdb->cf = CompoundFile_dup(fdb->cf);
+    nfdb->out_buffer_alloc = FASTADB_OUT_BUFFER_CHUNK_SIZE;
+    nfdb->out_buffer = g_malloc(sizeof(gchar)*nfdb->out_buffer_alloc);
+    nfdb->line_length = -1;
+    FastaDB_rewind(nfdb);
+    return nfdb;
+    }
+
 void FastaDB_close(FastaDB *fdb){
     g_assert(fdb);
     if(--fdb->ref_count)
@@ -238,6 +251,30 @@ CompoundFile_Pos FastaDB_find_next_start(FastaDB *fdb,
     return CompoundFile_ftell(fdb->cf)-1;
     }
 
+gboolean FastaDB_file_is_fasta(gchar *path){
+    register FILE *fp = fopen(path, "r");
+    register gint ch;
+    register gboolean result = FALSE;
+    if(!fp)
+        g_error("Could not open file [%s]", path);
+    while(((ch = getc(fp)) != EOF)){
+        if(isspace(ch))
+            continue;
+        else
+            if(ch == '>'){ /* Assume is fasta format */
+                result = TRUE;
+                break;
+                }
+        else
+            break;
+        }
+    fclose(fp);
+    return result;
+    }
+/* Assumes a file is fasta format
+ * if the first non-whitespace character is '>'
+ */
+
 gboolean FastaDB_is_finished(FastaDB *fdb){
     g_assert(fdb);
     return CompoundFile_is_finished(fdb->cf);
