@@ -3,7 +3,7 @@
 *  Library for manipulation of FASTA format databases            *
 *                                                                *
 *  Guy St.C. Slater..   mailto:guy@ebi.ac.uk                     *
-*  Copyright (C) 2000-2008.  All Rights Reserved.                *
+*  Copyright (C) 2000-2009.  All Rights Reserved.                *
 *                                                                *
 *  This source code is distributed under the terms of the        *
 *  GNU General Public License, version 3. See the file COPYING   *
@@ -73,6 +73,7 @@ typedef gboolean (*FastaDB_TraverseFunc)(FastaDB_Seq *fdbs,
              Alphabet *alphabet, gint chunk_id, gint chunk_total);
     FastaDB *FastaDB_open(gchar *path, Alphabet *alphabet);
     FastaDB *FastaDB_share(FastaDB *fdb);
+    FastaDB *FastaDB_dup(FastaDB *fdb); /* For use in a separate thread */
        void  FastaDB_close(FastaDB *fdb);
        void  FastaDB_rewind(FastaDB *fdb);
    gboolean  FastaDB_is_finished(FastaDB *fdb);
@@ -84,6 +85,9 @@ typedef gboolean (*FastaDB_TraverseFunc)(FastaDB_Seq *fdbs,
 CompoundFile_Pos  FastaDB_find_next_start(FastaDB *fdb,
                                           CompoundFile_Pos pos);
 
+    gboolean FastaDB_file_is_fasta(gchar *path);
+    /* Returns true if first non-whitespace character in file is '>' */
+
 typedef struct {
                 FastaDB *source;
   CompoundFile_Location *location;
