@@ -3,7 +3,7 @@
 *  Library for All-vs-All Fasta Database Comparisons             *
 *                                                                *
 *  Guy St.C. Slater..   mailto:guy@ebi.ac.uk                     *
-*  Copyright (C) 2000-2008.  All Rights Reserved.                *
+*  Copyright (C) 2000-2009.  All Rights Reserved.                *
 *                                                                *
 *  This source code is distributed under the terms of the        *
 *  GNU General Public License, version 3. See the file COPYING   *
@@ -21,7 +21,8 @@ FastaPipe *FastaPipe_create(FastaDB *query_fdb, FastaDB *target_fdb,
                FastaPipe_Boundary_Func term_func,
                FastaPipe_NextSeq_Func query_func,
                FastaPipe_NextSeq_Func target_func,
-               FastaDB_Mask mask, gboolean translate_both){
+               FastaDB_Mask mask, gboolean translate_both,
+               gboolean use_revcomp){
     register FastaPipe *fasta_pipe = g_new(FastaPipe, 1);
     g_assert(init_func);
     g_assert(prep_func);
@@ -37,12 +38,17 @@ FastaPipe *FastaPipe_create(FastaDB *query_fdb, FastaDB *target_fdb,
     fasta_pipe->target_func = target_func;
     fasta_pipe->mask = mask;
     fasta_pipe->is_full = FALSE;
-    fasta_pipe->revcomp_query
-        = (query_fdb->alphabet->type == Alphabet_Type_DNA);
-    fasta_pipe->revcomp_target
-        = (((query_fdb->alphabet->type == Alphabet_Type_PROTEIN)
-            && (target_fdb->alphabet->type == Alphabet_Type_DNA))
-          || translate_both);
+    if(use_revcomp){
+        fasta_pipe->revcomp_query
+            = (query_fdb->alphabet->type == Alphabet_Type_DNA);
+        fasta_pipe->revcomp_target
+            = (((query_fdb->alphabet->type == Alphabet_Type_PROTEIN)
+                && (target_fdb->alphabet->type == Alphabet_Type_DNA))
+              || translate_both);
+    } else {
+        fasta_pipe->revcomp_query = FALSE;
+        fasta_pipe->revcomp_target = FALSE;
+        }
     fasta_pipe->prev_query = NULL;
     fasta_pipe->prev_target = NULL;
     return fasta_pipe;
