@@ -3,7 +3,7 @@
 *  Library for All-vs-All Fasta Database Comparisons             *
 *                                                                *
 *  Guy St.C. Slater..   mailto:guy@ebi.ac.uk                     *
-*  Copyright (C) 2000-2008.  All Rights Reserved.                *
+*  Copyright (C) 2000-2009.  All Rights Reserved.                *
 *                                                                *
 *  This source code is distributed under the terms of the        *
 *  GNU General Public License, version 3. See the file COPYING   *
@@ -50,7 +50,8 @@ FastaPipe *FastaPipe_create(FastaDB *query_fdb, FastaDB *target_fdb,
                             FastaPipe_Boundary_Func term_func,
                             FastaPipe_NextSeq_Func query_func,
                             FastaPipe_NextSeq_Func target_func,
-                            FastaDB_Mask mask, gboolean translate_both);
+                            FastaDB_Mask mask, gboolean translate_both,
+                            gboolean use_revcomp);
 /* query_func can return TRUE to finish loading current pipeline
  * target_func can return TRUE to finish processing of the pipeline
  */
