@@ -3,7 +3,7 @@
 *  Library for All-vs-All Fasta Database Comparisons             *
 *                                                                *
 *  Guy St.C. Slater..   mailto:guy@ebi.ac.uk                     *
-*  Copyright (C) 2000-2008.  All Rights Reserved.                *
+*  Copyright (C) 2000-2009.  All Rights Reserved.                *
 *                                                                *
 *  This source code is distributed under the terms of the        *
 *  GNU General Public License, version 3. See the file COPYING   *
@@ -68,7 +68,7 @@ gint Argument_main(Argument *arg){
         fasta_pipe = FastaPipe_create(query_fdb, target_fdb,
                 test_init_func, test_prep_func, test_term_func,
                 test_query_func, test_target_func, FastaDB_Mask_ALL,
-                FALSE);
+                FALSE, TRUE);
         Alphabet_destroy(alphabet);
         while(FastaPipe_process(fasta_pipe, &count)){
             g_message("Processing pipeline");
