@@ -3,7 +3,7 @@
 *  Library for manipulation of exonerate index files             *
 *                                                                *
 *  Guy St.C. Slater..   mailto:guy@ebi.ac.uk                     *
-*  Copyright (C) 2000-2008.  All Rights Reserved.                *
+*  Copyright (C) 2000-2009.  All Rights Reserved.                *
 *                                                                *
 *  This source code is distributed under the terms of the        *
 *  GNU General Public License, version 3. See the file COPYING   *
@@ -26,7 +26,7 @@
 #include "noitree.h"
 
 #define INDEX_HEADER_MAGIC (('e' << 16)|('s' << 8)|('i'))
-#define INDEX_HEADER_VERSION 2
+#define INDEX_HEADER_VERSION 3
 
 static off_t Index_ftell(FILE *fp){
     return ftello(fp);
@@ -38,7 +38,7 @@ static off_t Index_ftell(FILE *fp){
 static off_t Index_fseek(FILE *fp, off_t offset, int whence){
     return fseeko(fp, offset, whence);
     }
-/* FIXME: need alternatives in absence of ftello() 
+/* FIXME: need alternatives in absence of fseeko()
  *        lseek(fileno(fp), offset, whence); ??
  */
 
@@ -78,7 +78,9 @@ static gsize Index_Word_get_address_list_space(Index_Word *index_word,
 
 static Index_Header *Index_Header_create(gint dataset_path_len,
                                          gboolean is_translated,
-                                         gint word_length, gint word_jump,
+                                         gint word_length,
+                                         gint word_jump,
+                                         gint word_ambiguity,
                                          gint saturate_threshold){
     register Index_Header *index_header = g_new0(Index_Header, 1);
     index_header->magic = INDEX_HEADER_MAGIC;
@@ -88,6 +90,7 @@ static Index_Header *Index_Header_create(gint dataset_path_len,
     /**/
     index_header->word_length = word_length;
     index_header->word_jump = word_jump;
+    index_header->word_ambiguity = word_ambiguity;
     index_header->saturate_threshold = saturate_threshold;
     return index_header;
     }
@@ -100,21 +103,23 @@ static void Index_Header_destroy(Index_Header *index_header){
 #if 0
 static void Index_Header_info(Index_Header *index_header){
     g_message("Index_Header:\n"
-              "    magic [%d]\n"
-              "    version [%d]\n"
-              "    type [%d]\n"
-              "    dataset_path_len [%d]\n"
+              "    magic [%lld]\n"
+              "    version [%lld]\n"
+              "    type [%lld]\n"
+              "    dataset_path_len [%lld]\n"
               "\n"
-              "    word_length [%d]\n"
-              "    word_jump [%d]\n"
-              "    saturate_threshold [%d]\n",
-           (gint)index_header->magic,
-           (gint)index_header->version,
-           (gint)index_header->type,
-           (gint)index_header->dataset_path_len,
-           (gint)index_header->word_length,
-           (gint)index_header->word_jump,
-           (gint)index_header->saturate_threshold);
+              "    word_length [%lld]\n"
+              "    word_jump [%lld]\n"
+              "    word_ambiguity [%lld]\n"
+              "    saturate_threshold [%lld]\n",
+           index_header->magic,
+           index_header->version,
+           index_header->type,
+           index_header->dataset_path_len,
+           index_header->word_length,
+           index_header->word_jump,
+           index_header->word_ambiguity,
+           index_header->saturate_threshold);
     return;
     }
 #endif /* 0 */
@@ -128,6 +133,7 @@ static void Index_Header_write(Index_Header *index_header, FILE *fp){
     /**/
     BitArray_write_int(index_header->word_length, fp);
     BitArray_write_int(index_header->word_jump, fp);
+    BitArray_write_int(index_header->word_ambiguity, fp);
     BitArray_write_int(index_header->saturate_threshold, fp);
     /**/
     return;
@@ -146,6 +152,7 @@ static Index_Header *Index_Header_read(FILE *fp){
     /**/
     index_header->word_length = BitArray_read_int(fp);
     index_header->word_jump  = BitArray_read_int(fp);
+    index_header->word_ambiguity = BitArray_read_int(fp);
     index_header->saturate_threshold = BitArray_read_int(fp);
     /* Index_Header_info(index_header); */
     return index_header;
@@ -186,7 +193,7 @@ typedef void (*Index_WordVisit_Func)(Index *index, Index_Strand *strand,
                                      gint seq_pos, gint leaf_id,
                                      gpointer user_data);
 
-static void Index_visit_seq_words(Index *index, Index_Strand *index_strand,
+static void Index_visit_seq_words_single(Index *index, Index_Strand *index_strand,
                                   gint seq_id, Sequence *seq,
                                   Index_WordVisit_Func iwvf, gint frame,
                                   gpointer user_data){
@@ -215,6 +222,81 @@ static void Index_visit_seq_words(Index *index, Index_Strand *index_strand,
     }
 /* FIXME: needs to work with softmasked sequences */
 
+#ifndef Swap
+#define Swap(x,y,temp) ((temp)=(x),(x)=(y),(y)=(temp))
+#endif /* Swap */
+
+static void Index_visit_seq_words_ambig(Index *index, Index_Strand *index_strand,
+                                  gint seq_id, Sequence *seq,
+                                  Index_WordVisit_Func iwvf, gint frame,
+                                  gpointer user_data){
+    register gint i, j, k, jump_ctr = 0, pos;
+    register gchar *ambig, *str = Sequence_get_str(seq);
+    register VFSM_Int state, next_state, leaf;
+    register VFSM_Int *temp_state_list,
+                      *curr_state_list = g_new0(VFSM_Int,
+                                                index->header->word_ambiguity),
+                      *next_state_list = g_new0(VFSM_Int,
+                                                index->header->word_ambiguity);
+    register gint curr_state_list_len = 1, next_state_list_len = 0;
+    for(i = 0; str[i]; i++){
+        if(jump_ctr--)
+            continue;
+        jump_ctr = index->header->word_jump - 1;
+        if((!(ambig = Alphabet_nt2ambig(str[i])))
+        || ((strlen(ambig) * curr_state_list_len)
+             > index->header->word_ambiguity)){
+            next_state_list_len = 0; /* Clear next state list */
+            curr_state_list_len = 1; /* Set single null state */
+            curr_state_list[0] = 0;
+            continue;
+            }
+        for(j = 0; j < curr_state_list_len; j++){
+            state = curr_state_list[j];
+            for(k = 0; ambig[k]; k++){
+                g_assert(index->vfsm->index[(guchar)ambig[k]]);
+                next_state = VFSM_change_state_M(index->vfsm,
+                                                 state, (guchar)ambig[k]);
+                if(next_state_list_len
+                && ((next_state == next_state_list[0])
+                 || (next_state == next_state_list[next_state_list_len-1])))
+                    break;
+                next_state_list[next_state_list_len++] = next_state;
+                if(VFSM_state_is_leaf(index->vfsm, next_state)){
+                    leaf = VFSM_state2leaf(index->vfsm, next_state);
+                    pos = i-(index->vfsm->depth-1);
+                    if(frame)
+                        pos = (pos * 3) + frame - 1;
+                    iwvf(index, index_strand, seq_id, pos, leaf, user_data);
+                    }
+                }
+            }
+        curr_state_list_len = next_state_list_len;
+        next_state_list_len = 0;
+        Swap(next_state_list, curr_state_list, temp_state_list);
+        }
+    g_free(str);
+    g_free(curr_state_list);
+    g_free(next_state_list);
+    return;
+    }
+/* FIXME: needs to work with softmasked sequences */
+/* FIXME: optimisation: remove strlen() call */
+
+static void Index_visit_seq_words(Index *index, Index_Strand *index_strand,
+                                  gint seq_id, Sequence *seq,
+                                  Index_WordVisit_Func iwvf, gint frame,
+                                  gpointer user_data){
+    if(index->header->word_ambiguity > 1)
+        Index_visit_seq_words_ambig(index, index_strand, seq_id, seq,
+                              iwvf, frame, user_data);
+    else
+        Index_visit_seq_words_single(index, index_strand, seq_id, seq,
+                              iwvf, frame, user_data);
+    return;
+    }
+/* FIXME: needs to work with softmasked sequences */
+
 static void Index_visit_words(Index *index, Index_Strand *index_strand,
                               Index_WordVisit_Func iwvf,
                               gboolean is_forward, gpointer user_data){
@@ -292,7 +374,8 @@ static void Index_desaturate(Index *index, Index_Strand *index_strand){
             removed_instances += index_strand->word_table[i];
             index_strand->word_table[i] = -1;
             }
-    g_message("Removed [%lld] words, [%lld] instances",
+    g_message("Removed [%" CUSTOM_GUINT64_FORMAT
+              "] words, [%" CUSTOM_GUINT64_FORMAT "] instances",
               removed_words, removed_instances);
     return;
     }
@@ -663,8 +746,8 @@ static void Index_Strand_destroy(Index_Strand *index_strand){
     return;
     }
 
-Index *Index_create(Dataset *dataset, gboolean is_translated,
-                    gint word_length, gint word_jump, gint saturate_threshold,
+Index *Index_create(Dataset *dataset, gboolean is_translated, gint word_length,
+                    gint word_jump, gint word_ambiguity, gint saturate_threshold,
                     gchar *index_path, gchar *dataset_path, gint memory_limit){
     register Index *index = g_new0(Index, 1);
     register gchar *member;
@@ -684,7 +767,8 @@ Index *Index_create(Dataset *dataset, gboolean is_translated,
     index->dataset = Dataset_share(dataset);
     index->header = Index_Header_create(strlen(dataset_path),
                                         is_translated, word_length,
-                                        word_jump, saturate_threshold);
+                                        word_jump, word_ambiguity,
+                                        saturate_threshold);
     if(is_translated){
         alphabet = Alphabet_create(Alphabet_Type_PROTEIN, FALSE);
         member = (gchar*)alphabet->member;
@@ -781,6 +865,24 @@ void Index_destroy(Index *index){
     return;
     }
 
+void Index_info(Index *index){
+    g_print("Sequence Index:\n"
+            "--------------\n"
+            "                version: %d\n"
+            "                   type: %s\n"
+            "            word length: %d\n"
+            "              word jump: %d\n"
+            "         word ambiguity: %d\n"
+            "     saturate threshold: %d\n\n",
+            (gint)index->header->version,
+            (index->header->type & 1) ? "translated" : "normal",
+            (gint)index->header->word_length,
+            (gint)index->header->word_jump,
+            (gint)index->header->word_ambiguity,
+            (gint)index->header->saturate_threshold);
+    return;
+    }
+
 static Index_Word *Index_Strand_read_word_list(Index *index,
                                                Index_Strand *index_strand){
     register Index_Word *word_list = g_new(Index_Word,
