@@ -3,7 +3,7 @@
 *  Library for manipulation of exonerate index files             *
 *                                                                *
 *  Guy St.C. Slater..   mailto:guy@ebi.ac.uk                     *
-*  Copyright (C) 2000-2008.  All Rights Reserved.                *
+*  Copyright (C) 2000-2009.  All Rights Reserved.                *
 *                                                                *
 *  This source code is distributed under the terms of the        *
 *  GNU General Public License, version 3. See the file COPYING   *
@@ -60,6 +60,7 @@ typedef struct {
     /**/
     guint64  word_length;
     guint64  word_jump;
+    guint64  word_ambiguity;
     guint64  saturate_threshold;
 } Index_Header;
 
@@ -117,11 +118,12 @@ typedef struct {
 
 /**/
 
-   Index *Index_create(Dataset *dataset, gboolean is_translated,
-                       gint word_length, gint word_jump, gint saturate_threshold,
+   Index *Index_create(Dataset *dataset, gboolean is_translated, gint word_length,
+                       gint word_jump, gint word_ambiguity, gint saturate_threshold,
                        gchar *index_path, gchar *dataset_path, gint memory_limit);
    Index *Index_share(Index *index);
     void  Index_destroy(Index *index);
+    void  Index_info(Index *index);
    Index *Index_open(gchar *path);
  guint64  Index_memory_usage(Index *index);
     void  Index_preload_index(Index *index);
@@ -150,5 +152,5 @@ GPtrArray *Index_get_HSPsets_geneseed(Index *index, HSP_Param *hsp_param,
 }
 #endif /* __cplusplus */
 
-#endif /* INCLUDED_DATASET_H */
+#endif /* INCLUDED_INDEX_H */
 
