@@ -3,7 +3,7 @@
 *  Library for command line argument processing                  *
 *                                                                *
 *  Guy St.C. Slater..   mailto:guy@ebi.ac.uk                     *
-*  Copyright (C) 2000-2008.  All Rights Reserved.                *
+*  Copyright (C) 2000-2009.  All Rights Reserved.                *
 *                                                                *
 *  This source code is distributed under the terms of the        *
 *  GNU General Public License, version 3. See the file COPYING   *
@@ -173,8 +173,6 @@ static void Argument_show_version(Argument *arg){
     g_print("Built on %s\n", __DATE__);
     if(strlen(branch) >= 10)
         g_print("Branch: %.*s\n", (gint)(strlen(branch)-9), branch+7);
-    else
-        g_print("Branch: unnamed branch\n");
     return;
     }
 
@@ -302,8 +300,13 @@ static void Argument_error_handler(const gchar *log_domain,
  */
 
 int main(int argc, char **argv){
-    register Argument *arg = Argument_create(argc, argv);
+    register Argument *arg;
     register gint retval;
+#ifdef USE_PTHREADS
+    if(!g_thread_supported())
+        g_thread_init(NULL);
+#endif /* USE_PTHREADS */
+    arg = Argument_create(argc, argv);
     g_log_set_handler(NULL, G_LOG_LEVEL_ERROR|G_LOG_FLAG_FATAL,
                       Argument_error_handler, arg);
     g_assert(Argument_assertion_warning());
