@@ -3,7 +3,7 @@
 *  Analysis module for exonerate                                 *
 *                                                                *
 *  Guy St.C. Slater..   mailto:guy@ebi.ac.uk                     *
-*  Copyright (C) 2000-2008.  All Rights Reserved.                *
+*  Copyright (C) 2000-2009.  All Rights Reserved.                *
 *                                                                *
 *  This source code is distributed under the terms of the        *
 *  GNU General Public License, version 3. See the file COPYING   *
@@ -17,12 +17,17 @@
 #include "ungapped.h"
 #include "compoundfile.h"
 #include "hspset.h"
+#include "lineparse.h"
 
 #include <stdlib.h> /* For atoi() */
 #include <unistd.h> /* For sleep() */
 #include <string.h> /* For strcmp() */
 #include <ctype.h>  /* For isdigit() */
 
+#include <sys/types.h>  /* For stat() */
+#include <sys/stat.h>   /* For stat() */
+#include <unistd.h>     /* For stat() */
+
 Analysis_ArgumentSet *Analysis_ArgumentSet_create(Argument *arg){
     register ArgumentSet *as;
     static Analysis_ArgumentSet aas;
@@ -34,6 +39,9 @@ Analysis_ArgumentSet *Analysis_ArgumentSet_create(Argument *arg){
         ArgumentSet_add_option(as, 'B', "bigseq", NULL,
             "Allow rapid comparison between big sequences", "FALSE",
             Argument_parse_boolean, &aas.use_bigseq);
+        ArgumentSet_add_option(as, 'r', "revcomp", NULL,
+            "Also search reverse complement of query and target", "TRUE",
+            Argument_parse_boolean, &aas.use_revcomp);
         ArgumentSet_add_option(as, '\0', "forcescan", "[q|t]",
             "Force FSM scan on query or target sequences", "none",
             Argument_parse_string, &aas.force_scan);
@@ -45,6 +53,9 @@ Analysis_ArgumentSet *Analysis_ArgumentSet_create(Argument *arg){
         ArgumentSet_add_option(as, 0, "customserver", "command",
             "Custom command to send non-standard server", "NULL",
             Argument_parse_string, &aas.custom_server_command);
+        ArgumentSet_add_option(as, 'c', "cores", "number",
+            "Number of cores/CPUs/threads for alignment computation", "1",
+            Argument_parse_int, &aas.thread_count);
         Argument_absorb_ArgumentSet(arg, as);
         }
     return &aas;
@@ -52,10 +63,43 @@ Analysis_ArgumentSet *Analysis_ArgumentSet_create(Argument *arg){
 
 /**/
 
+typedef struct {
+           GAM *gam;
+    Comparison *comparison;
+} Analysis_HeuristicJob;
+
+static Analysis_HeuristicJob *Analysis_HeuristicJob_create(GAM *gam,
+                                       Comparison *comparison){
+    register Analysis_HeuristicJob *ahj = g_new(Analysis_HeuristicJob, 1);
+    ahj->gam = GAM_share(gam);
+    ahj->comparison = Comparison_share(comparison);
+    return ahj;
+    }
+
+static void Analysis_HeuristicJob_destroy(Analysis_HeuristicJob *ahj){
+    Comparison_destroy(ahj->comparison);
+    GAM_destroy(ahj->gam);
+    g_free(ahj);
+    return;
+    }
+
+static void Analysis_HeuristicJob_run(gpointer data){
+    register Analysis_HeuristicJob *ahj = data;
+    register GAM_Result *gam_result
+           = GAM_Result_heuristic_create(ahj->gam, ahj->comparison);
+    if(gam_result){
+        GAM_Result_submit(gam_result);
+        GAM_Result_destroy(gam_result);
+        }
+    Analysis_HeuristicJob_destroy(ahj);
+    return;
+    }
+
 static void Analysis_report_func(Comparison *comparison,
                                  gpointer user_data){
     register Analysis *analysis = user_data;
     register GAM_Result *gam_result;
+    register Analysis_HeuristicJob *ahj;
     g_assert(Comparison_has_hsps(comparison));
     if(analysis->scan_query){
         /* Swap back query and target after a query scan */
@@ -70,15 +114,21 @@ static void Analysis_report_func(Comparison *comparison,
             Comparison_revcomp(comparison);
         }
     if(Model_Type_is_gapped(analysis->gam->gas->type)){
-        gam_result = GAM_Result_heuristic_create(analysis->gam,
-                                                 comparison);
+        ahj = Analysis_HeuristicJob_create(analysis->gam, comparison);
+#ifdef USE_PTHREADS
+        g_assert(analysis->job_queue);
+        /* FIXME: need to use priority here */
+        JobQueue_submit(analysis->job_queue, Analysis_HeuristicJob_run, ahj, 2);
+#else /* USE_PTHREADS */
+        Analysis_HeuristicJob_run(ahj);
+#endif /* USE_PTHREADS */
     } else {
         gam_result = GAM_Result_ungapped_create(analysis->gam,
                                                 comparison);
-        }
-    if(gam_result){
-        GAM_Result_submit(gam_result);
-        GAM_Result_destroy(gam_result);
+        if(gam_result){
+            GAM_Result_submit(gam_result);
+            GAM_Result_destroy(gam_result);
+            }
         }
     return;
     }
@@ -108,7 +158,7 @@ static void Analysis_FastaPipe_Pair_term_func(gpointer user_data){
 /* Called after query pipeline analysis */
 
 static gboolean Analysis_FastaPipe_Pair_query_func(FastaDB_Seq *fdbs,
-                                                   gpointer user_data){
+                                                  gpointer user_data){
     register Analysis *analysis = user_data;
     g_assert(!analysis->curr_query);
     /*
@@ -139,22 +189,63 @@ static void Analysis_BSAM_compare(Analysis *analysis,
 
 /**/
 
+typedef struct {
+         GAM *gam;
+    Sequence *query;
+    Sequence *target;
+} Analysis_ExhaustiveJob;
+
+static Analysis_ExhaustiveJob *Analysis_ExhaustiveJob_create(GAM *gam,
+                                        Sequence *query, Sequence *target){
+    register Analysis_ExhaustiveJob *aej = g_new(Analysis_ExhaustiveJob, 1);
+    aej->gam = GAM_share(gam);
+    aej->query = Sequence_share(query);
+    aej->target = Sequence_share(target);
+    return aej;
+    }
+
+static void Analysis_ExhaustiveJob_destroy(Analysis_ExhaustiveJob *aej){
+    GAM_destroy(aej->gam);
+    Sequence_destroy(aej->query);
+    Sequence_destroy(aej->target);
+    g_free(aej);
+    return;
+    }
+
+static void Analysis_ExhaustiveJob_run(gpointer data){
+    register Analysis_ExhaustiveJob *aej = data;
+    register GAM_Result *gam_result;
+    gam_result = GAM_Result_exhaustive_create(aej->gam,
+                                              aej->query, aej->target);
+    if(gam_result){
+        GAM_Result_submit(gam_result);
+        GAM_Result_destroy(gam_result);
+        }
+    Analysis_ExhaustiveJob_destroy(aej);
+    return;
+    }
+
 static void Analysis_Pair_compare(Analysis *analysis,
                                   FastaDB_Seq *fdbs){
-    register GAM_Result *gam_result;
+    register Analysis_ExhaustiveJob *aej;
     if(analysis->aas->use_exhaustive){
-        gam_result = GAM_Result_exhaustive_create(analysis->gam,
-                         analysis->curr_query->seq, fdbs->seq);
-        if(gam_result){
-            GAM_Result_submit(gam_result);
-            GAM_Result_destroy(gam_result);
-            }
+        aej = Analysis_ExhaustiveJob_create(analysis->gam,
+                                            analysis->curr_query->seq,
+                                            fdbs->seq);
+#ifdef USE_PTHREADS
+        g_assert(analysis->job_queue);
+        JobQueue_submit(analysis->job_queue, Analysis_ExhaustiveJob_run, aej, 1);
+#else /* USE_PTHREADS */
+        Analysis_ExhaustiveJob_run(aej);
+#endif /* USE_PTHREADS */
     } else {
         Analysis_BSAM_compare(analysis, analysis->curr_query, fdbs);
         }
     return;
     }
 
+/**/
+
 static gboolean Analysis_FastaPipe_Pair_target_func(FastaDB_Seq *fdbs,
                                                     gpointer user_data){
     register Analysis *analysis = user_data;
@@ -420,6 +511,24 @@ static void Analysis_Client_set_param(Analysis_Client *aclient, GAM *gam){
     g_free(msg);
     g_free(reply);
     /**/
+    msg = g_strdup_printf("set param dnawordlimit %d",
+                          has->dna_word_limit);
+    reply = Analysis_Client_send(aclient, msg, "ok:", FALSE);
+    g_free(msg);
+    g_free(reply);
+    /**/
+    msg = g_strdup_printf("set param proteinwordlimit %d",
+                          has->protein_word_limit);
+    reply = Analysis_Client_send(aclient, msg, "ok:", FALSE);
+    g_free(msg);
+    g_free(reply);
+    /**/
+    msg = g_strdup_printf("set param codonwordlimit %d",
+                          has->codon_word_limit);
+    reply = Analysis_Client_send(aclient, msg, "ok:", FALSE);
+    g_free(msg);
+    g_free(reply);
+    /**/
     msg = g_strdup_printf("set param dnahspdropoff %d",
                           has->dna_hsp_dropoff);
     reply = Analysis_Client_send(aclient, msg, "ok:", FALSE);
@@ -465,6 +574,10 @@ static void Analysis_Client_set_param(Analysis_Client *aclient, GAM *gam){
     return;
     }
 
+static void Analysis_Client_info(Analysis_Client *aclient){
+    return;
+    }
+
 static Analysis_Client *Analysis_Client_create(gchar *path, gint verbosity){
     register Analysis_Client *aclient;
     register SocketClient *sc = Analysis_Client_connect(path);
@@ -473,6 +586,7 @@ static Analysis_Client *Analysis_Client_create(gchar *path, gint verbosity){
     if(!sc)
         return NULL;
     aclient = g_new(Analysis_Client, 1);
+    aclient->ref_count = 1;
     aclient->sc = sc;
     aclient->verbosity = verbosity;
     aclient->probe_fdb = NULL;
@@ -508,12 +622,20 @@ static Analysis_Client *Analysis_Client_create(gchar *path, gint verbosity){
     g_free(dbinfo);
     aclient->curr_query = NULL;
     aclient->seq_cache = g_new0(Sequence*, aclient->num_seqs);
+    Analysis_Client_info(aclient);
+    return aclient;
+    }
+
+static Analysis_Client *Analysis_Client_share(Analysis_Client *aclient){
+    aclient->ref_count++;
     return aclient;
     }
 
 static void Analysis_Client_destroy(Analysis_Client *aclient){
     register gint i;
     register Sequence *seq;
+    if(--aclient->ref_count)
+        return;
     if(aclient->curr_query)
         Sequence_destroy(aclient->curr_query);
     for(i = 0; i < aclient->num_seqs; i++){
@@ -533,7 +655,7 @@ static void Analysis_Client_destroy(Analysis_Client *aclient){
 static void Analysis_Client_set_probe_fdb(Analysis_Client *aclient,
                                           FastaDB *probe_fdb){
     g_assert(!aclient->probe_fdb);
-    aclient->probe_fdb = FastaDB_share(probe_fdb);
+    aclient->probe_fdb = FastaDB_dup(probe_fdb);
     return;
     }
 
@@ -592,13 +714,14 @@ typedef struct {
 static Analysis_Client_Key *Analysis_Client_Key_create(Analysis_Client *aclient,
                                                    gint target_id, gint seq_len){
     register Analysis_Client_Key *key = g_new(Analysis_Client_Key, 1);
-    key->aclient = aclient;
+    key->aclient = Analysis_Client_share(aclient);
     key->target_id = target_id;
     key->seq_len = seq_len;
     return key;
     }
 
 static void Analysis_Client_Key_destroy(Analysis_Client_Key *key){
+    Analysis_Client_destroy(key->aclient);
     g_free(key);
     return;
     }
@@ -848,7 +971,8 @@ static void Analysis_Client_process_query(Analysis_Client *aclient,
                                           Analysis *analysis,
                                           Sequence *query,
                                           gboolean swap_chains,
-                                          gboolean revcomp_target){
+                                          gboolean revcomp_target,
+                                          gint priority){
     Analysis_Client_set_query(aclient, query);
     Analysis_Client_get_hsp_sets(aclient, analysis, swap_chains, revcomp_target);
     /* Revcomp query if DNA */
@@ -861,19 +985,20 @@ static void Analysis_Client_process_query(Analysis_Client *aclient,
     }
 
 static void Analysis_Client_process(Analysis_Client *aclient, Analysis *analysis,
-                                    gboolean swap_chains){
+                                    gboolean swap_chains, gint priority){
     register FastaDB_Seq *fdbs;
     /* FIXME: need to check for appropriate database type */
     while((fdbs = FastaDB_next(aclient->probe_fdb, FastaDB_Mask_ALL))){
         Analysis_Client_process_query(aclient, analysis, fdbs->seq,
-                                      swap_chains, FALSE);
+                                      swap_chains, FALSE, priority);
         /* Revcomp target if protein vs DNA or translate_both */
         if(((aclient->curr_query->alphabet->type == Alphabet_Type_PROTEIN)
           && (aclient->server_alphabet->type == Alphabet_Type_DNA))
            || analysis->gam->translate_both){
             Analysis_Client_revcomp_target(aclient);
             Analysis_Client_process_query(aclient, analysis,
-                                          fdbs->seq, swap_chains, TRUE);
+                                          fdbs->seq, swap_chains, TRUE,
+                                          priority);
             Analysis_Client_revcomp_target(aclient);
             }
         FastaDB_Seq_destroy(fdbs);
@@ -883,6 +1008,163 @@ static void Analysis_Client_process(Analysis_Client *aclient, Analysis *analysis
 
 /**/
 
+static Analysis_Server *Analysis_Server_create(Analysis_Builder *ab,
+                                               gchar *name, gint priority){
+    register Analysis_Server *as = g_new(Analysis_Server, 1);
+    as->name = g_strdup(name);
+    as->ab = ab;
+    as->priority = priority;
+    return as;
+    }
+
+static void Analysis_Server_destroy(Analysis_Server *as){
+    g_free(as->name);
+    g_free(as);
+    return;
+    }
+
+/**/
+
+static Analysis_Builder *Analysis_Builder_create(GPtrArray *path_list,
+                                                 Analysis *analysis,
+                                                 gint verbosity){
+    register Analysis_Builder *ab;
+    register gint i;
+    register Analysis_Client *ac;
+    register Analysis_Server *as;
+    g_assert(path_list->len);
+    ac = Analysis_Client_create(path_list->pdata[0], analysis->verbosity);
+    if(!ac)
+        return NULL;
+    /**/
+    ab = g_new(Analysis_Builder, 1);
+    ab->verbosity = verbosity;
+    ab->server_list = g_ptr_array_new();
+    ab->server_type = ac->server_alphabet->type;
+    ab->probe_fdb = NULL;
+    ab->analysis = analysis;
+    Analysis_Client_destroy(ac);
+    for(i = 0; i < path_list->len; i++){
+        as = Analysis_Server_create(ab, path_list->pdata[i], i);
+        g_ptr_array_add(ab->server_list, as);
+        }
+    return ab;
+    }
+
+static void Analysis_Builder_destroy(Analysis_Builder *ab){
+    register gint i;
+    register Analysis_Server *as;
+    for(i = 0; i < ab->server_list->len; i++){
+        as = ab->server_list->pdata[i];
+        Analysis_Server_destroy(as);
+        }
+    g_ptr_array_free(ab->server_list, TRUE);
+    if(ab->probe_fdb)
+        FastaDB_close(ab->probe_fdb);
+    g_free(ab);
+    return;
+    }
+
+static void Analysis_Server_run(gpointer data){
+    register Analysis_Server *server = data;
+    register Analysis_Client *client
+                = Analysis_Client_create(server->name,
+                                         server->ab->analysis->verbosity);
+    if(!client)
+        g_error("Could not connect to server [%s]", server->name);
+    Analysis_Client_set_param(client, server->ab->analysis->gam);
+    Analysis_Client_set_probe_fdb(client, server->ab->probe_fdb);
+    Analysis_Client_process(client, server->ab->analysis,
+                            server->ab->swap_chains, server->priority);
+    /**/
+    Analysis_Client_destroy(client);
+    return;
+    }
+
+static void Analysis_Builder_process(Analysis_Builder *ab,
+                                     Analysis *analysis, gboolean swap_chains){
+    register gint i;
+    register Analysis_Server *as;
+    ab->swap_chains = swap_chains;
+    for(i = 0; i < ab->server_list->len; i++){
+        as = ab->server_list->pdata[i];
+#ifdef USE_PTHREADS
+        g_assert(analysis->job_queue);
+        JobQueue_submit(analysis->job_queue, Analysis_Server_run, as, i);
+#else /* USE_PTHREADS */
+        Analysis_Server_run(as);
+#endif /* USE_PTHREADS */
+        }
+    return;
+    }
+
+static void Analysis_Builder_set_probe_fdb(Analysis_Builder *ab,
+                                           FastaDB *probe_fdb){
+    g_assert(!ab->probe_fdb);
+    ab->probe_fdb = FastaDB_share(probe_fdb);
+    return;
+    }
+
+/**/
+
+static void Analysis_path_list_destroy(GPtrArray *path_list){
+    register gint i;
+    register gchar *path;
+    for(i = 0; i < path_list->len; i++){
+        path = path_list->pdata[i];
+        g_free(path);
+        }
+    g_ptr_array_free(path_list, TRUE);
+    return;
+    }
+
+static GPtrArray *Analysis_FOSN_expand_path_list(GPtrArray *path_list){
+    register gint i;
+    register gchar *path, *npath;
+    struct stat buf;
+    register gboolean expanded_list = FALSE;
+    register GPtrArray *npath_list = g_ptr_array_new();
+    register FILE *fp;
+    register LineParse *lp;
+    for(i = 0; i < path_list->len; i++){
+        path = path_list->pdata[i];
+        if((stat(path, &buf))              /* Cannot read file (ie. servername) */
+        || (S_ISDIR(buf.st_mode))          /* Is directory */
+        || (FastaDB_file_is_fasta(path))){ /* 1st non-whitespace char is '>' */
+            npath = g_strdup(path);
+            g_ptr_array_add(npath_list, npath);
+        } else { /* Assume to be fosn */
+            fp = fopen(path, "r");
+            if(!fp)
+                g_error("Could not open FOSN file [%s]", path);
+            lp = LineParse_create(fp);
+            while(LineParse_line(lp) != EOF){
+                g_strstrip(lp->line->str); /* strip whitespace */
+                if(lp->line->str[0]){
+                    npath = g_strdup(lp->line->str);
+                    g_ptr_array_add(npath_list, npath);
+                    expanded_list = TRUE;
+                    }
+                }
+            LineParse_destroy(lp);
+            fclose(fp);
+            }
+        }
+    if(!expanded_list){
+        for(i = 0; i < npath_list->len; i++){
+            npath = npath_list->pdata[i];
+            g_free(npath);
+            }
+        g_ptr_array_free(npath_list, TRUE);
+        return NULL;
+        }
+    return npath_list;
+    }
+/* If members of the path list are files
+ * where the first non-whitespace character is not '>',
+ * it is assumed to be a FOSN, and parsed to expand the path_list.
+ */
+
 Analysis *Analysis_create(
               GPtrArray *query_path_list, Alphabet_Type query_type,
               gint query_chunk_id, gint query_chunk_total,
@@ -899,26 +1181,33 @@ Analysis *Analysis_create(
                        *codon_hsp_param;
     register Match_ArgumentSet *mas = Match_ArgumentSet_create(NULL);
     register gboolean use_horizon;
+    register GPtrArray *expanded_query_path_list = NULL,
+                       *expanded_target_path_list = NULL;
     g_assert(query_path_list);
     g_assert(target_path_list);
     g_assert(query_path_list->len);
     g_assert(target_path_list->len);
     analysis->aas = Analysis_ArgumentSet_create(NULL);
-
     analysis->verbosity = verbosity;
+    analysis->job_queue = JobQueue_create(analysis->aas->thread_count);
+    /* Expand FOSN paths */
+    expanded_query_path_list = Analysis_FOSN_expand_path_list(
+                                                    query_path_list);
+    expanded_target_path_list = Analysis_FOSN_expand_path_list(
+                                                    target_path_list);
+    if(expanded_query_path_list)
+        query_path_list = expanded_query_path_list;
+    if(expanded_target_path_list)
+        target_path_list = expanded_target_path_list;
     /**/
-    if(query_path_list->len == 1)
-        analysis->query_ac
-                = Analysis_Client_create((gchar*)query_path_list->pdata[0],
-                                         verbosity);
-    if(target_path_list->len == 1)
-        analysis->target_ac
-                = Analysis_Client_create((gchar*)target_path_list->pdata[0],
-                                       verbosity);
+    analysis->query_builder = Analysis_Builder_create(query_path_list,
+                                                     analysis, verbosity);
+    analysis->target_builder = Analysis_Builder_create(target_path_list,
+                                                     analysis, verbosity);
     /**/
     if(query_type == Alphabet_Type_UNKNOWN){
-        if(analysis->query_ac){
-            query_type = analysis->query_ac->server_alphabet->type;
+        if(analysis->query_builder){
+            query_type = analysis->query_builder->server_type;
         } else {
             query_type = FastaDB_guess_type(
                               (gchar*)query_path_list->pdata[0]);
@@ -928,8 +1217,8 @@ Analysis *Analysis_create(
             }
         }
     if(target_type == Alphabet_Type_UNKNOWN){
-        if(analysis->target_ac){
-            target_type = analysis->target_ac->server_alphabet->type;
+        if(analysis->target_builder){
+            target_type = analysis->target_builder->server_type;
         } else {
             target_type = FastaDB_guess_type(
                               (gchar*)target_path_list->pdata[0]);
@@ -953,11 +1242,6 @@ Analysis *Analysis_create(
                                analysis->aas->use_exhaustive,
                                verbosity);
     /**/
-    if(analysis->query_ac)
-        Analysis_Client_set_param(analysis->query_ac, analysis->gam);
-    if(analysis->target_ac)
-        Analysis_Client_set_param(analysis->target_ac, analysis->gam);
-    /**/
     Analysis_find_matches(analysis, &dna_match, &protein_match,
                                     &codon_match);
     match = dna_match;
@@ -966,14 +1250,14 @@ Analysis *Analysis_create(
     if(!match)
         match = codon_match;
     g_assert(match);
-    if(!analysis->query_ac)
+    if(!analysis->query_builder)
         query_fdb = FastaDB_open_list_with_limit(query_path_list,
                 match->query->alphabet, query_chunk_id, query_chunk_total);
-    if(!analysis->target_ac)
+    if(!analysis->target_builder)
        target_fdb = FastaDB_open_list_with_limit(target_path_list,
                match->target->alphabet, target_chunk_id, target_chunk_total);
     if(analysis->aas->use_exhaustive){
-        if(analysis->query_ac || analysis->target_ac) /* FIXME: ni */
+        if(analysis->query_builder || analysis->target_builder) /* FIXME: ni */
             g_error("Exhaustive alignment against server not implemented");
         analysis->fasta_pipe = FastaPipe_create(
                                   query_fdb, target_fdb,
@@ -983,12 +1267,13 @@ Analysis *Analysis_create(
                                   Analysis_FastaPipe_Pair_query_func,
                                   Analysis_FastaPipe_Pair_target_func,
                                   FastaDB_Mask_ALL,
-                                  analysis->gam->translate_both);
+                                  analysis->gam->translate_both,
+                                  analysis->aas->use_revcomp);
         analysis->curr_query = NULL;
     } else { /* Not exhaustive */
         use_horizon = (analysis->aas->use_bigseq
-                    || analysis->query_ac
-                    || analysis->target_ac)?FALSE:TRUE;
+                    || analysis->query_builder
+                    || analysis->target_builder)?FALSE:TRUE;
         dna_hsp_param = dna_match
                       ? HSP_Param_create(dna_match, use_horizon)
                       : NULL;
@@ -1026,15 +1311,17 @@ Analysis *Analysis_create(
                     = analysis->gam->gas->threshold;
             }
         /* Don't need HSP horizon for bigseq comparison */
-        if(analysis->query_ac || analysis->target_ac){
-            if(analysis->query_ac && analysis->target_ac)
+        if(analysis->query_builder || analysis->target_builder){
+            if(analysis->query_builder && analysis->target_builder)
                 g_error("Server vs server comparison not impelemented");
             analysis->fasta_pipe = NULL;
-            if(analysis->query_ac){
-                Analysis_Client_set_probe_fdb(analysis->query_ac, target_fdb);
+            if(analysis->query_builder){
+                Analysis_Builder_set_probe_fdb(analysis->query_builder,
+                                              target_fdb);
             } else {
-                g_assert(analysis->target_ac);
-                Analysis_Client_set_probe_fdb(analysis->target_ac, query_fdb);
+                g_assert(analysis->target_builder);
+                Analysis_Builder_set_probe_fdb(analysis->target_builder,
+                                               query_fdb);
                 }
         } else {
             if(analysis->aas->use_bigseq){
@@ -1049,7 +1336,8 @@ Analysis *Analysis_create(
                                       Analysis_FastaPipe_Pair_query_func,
                                       Analysis_FastaPipe_Pair_target_func,
                                       FastaDB_Mask_ALL,
-                                      analysis->gam->translate_both);
+                                      analysis->gam->translate_both,
+                                      analysis->aas->use_revcomp);
                 analysis->curr_query = NULL;
             } else { /* Use Seeder */
                 analysis->scan_query = Analysis_decide_scan_query(query_fdb,
@@ -1076,7 +1364,8 @@ Analysis *Analysis_create(
                     Analysis_FastaPipe_Seeder_term_func,
                     Analysis_FastaPipe_Seeder_query_func,
                     Analysis_FastaPipe_Seeder_target_func,
-                    FastaDB_Mask_ALL, analysis->gam->translate_both);
+                    FastaDB_Mask_ALL, analysis->gam->translate_both,
+                    analysis->aas->use_revcomp);
                 }
             }
         }
@@ -1085,10 +1374,15 @@ Analysis *Analysis_create(
     if(target_fdb)
         FastaDB_close(target_fdb);
     /**/
+    if(expanded_query_path_list)
+        Analysis_path_list_destroy(expanded_query_path_list);
+    if(expanded_target_path_list)
+        Analysis_path_list_destroy(expanded_target_path_list);
     return analysis;
     }
 
 void Analysis_destroy(Analysis *analysis){
+    JobQueue_destroy(analysis->job_queue);
     if(analysis->fasta_pipe)
         FastaPipe_destroy(analysis->fasta_pipe);
     if(analysis->curr_query)
@@ -1099,23 +1393,31 @@ void Analysis_destroy(Analysis *analysis){
         BSAM_destroy(analysis->bsam);
     if(analysis->comparison_param)
         Comparison_Param_destroy(analysis->comparison_param);
+    /*
     if(analysis->query_ac)
         Analysis_Client_destroy(analysis->query_ac);
     if(analysis->target_ac)
         Analysis_Client_destroy(analysis->target_ac);
+    */
+    if(analysis->query_builder)
+        Analysis_Builder_destroy(analysis->query_builder);
+    if(analysis->target_builder)
+        Analysis_Builder_destroy(analysis->target_builder);
     GAM_destroy(analysis->gam);
     g_free(analysis);
     return;
     }
 
 void Analysis_process(Analysis *analysis){
-    if(analysis->query_ac){
-        Analysis_Client_process(analysis->query_ac, analysis, TRUE);
-    } else if(analysis->target_ac){
-        Analysis_Client_process(analysis->target_ac, analysis, FALSE);
+    if(analysis->query_builder){
+        Analysis_Builder_process(analysis->query_builder, analysis, TRUE);
+    } else if(analysis->target_builder){
+        Analysis_Builder_process(analysis->target_builder, analysis, FALSE);
     } else {
         while(FastaPipe_process(analysis->fasta_pipe, analysis));
         }
+    if(analysis->job_queue)
+        JobQueue_complete(analysis->job_queue);
     GAM_report(analysis->gam);
     return;
     }
