@@ -3,7 +3,7 @@
 *  GAM: Gapped Alignment Manager                                 *
 *                                                                *
 *  Guy St.C. Slater..   mailto:guy@ebi.ac.uk                     *
-*  Copyright (C) 2000-2008.  All Rights Reserved.                *
+*  Copyright (C) 2000-2009.  All Rights Reserved.                *
 *                                                                *
 *  This source code is distributed under the terms of the        *
 *  GNU General Public License, version 3. See the file COPYING   *
@@ -162,32 +162,18 @@ static gboolean GAM_StoredResult_compare(gpointer low,
 
 static void GAM_display_alignment(GAM *gam,
             Alignment *alignment, Sequence *query, Sequence *target,
-            gint result_id, gint rank, gpointer user_data, FILE *fp);
-
-#if 0
-static FILE *GAM_bestn_tmp_file = NULL;
-
-static void GAM_StoredResult_print(const gchar *string){
-    g_assert(GAM_bestn_tmp_file);
-    fprintf(GAM_bestn_tmp_file, "%s", string);
-    return;
-    }
-#endif /* 0 */
+            gint result_id, gint rank,
+            gpointer user_data, gpointer self_data, FILE *fp);
 
 static GAM_StoredResult *GAM_StoredResult_create(GAM *gam,
                          Sequence *query, Sequence *target,
-                         Alignment *alignment, gpointer user_data){
+                         Alignment *alignment,
+                         gpointer user_data, gpointer self_data){
     register GAM_StoredResult *gsr = g_new(GAM_StoredResult, 1);
-    /* register GPrintFunc prev_print_func; */
     gsr->score = alignment->score;
     gsr->pos = ftell(gam->bestn_tmp_file);
-    /*
-    GAM_bestn_tmp_file = gam->bestn_tmp_file;
-    prev_print_func = g_set_print_handler(GAM_StoredResult_print);
-    */
     GAM_display_alignment(gam, alignment, query, target,
-                          0, -1, user_data, gam->bestn_tmp_file);
-    /* g_set_print_handler(prev_print_func); */
+                          0, -1, user_data, self_data, gam->bestn_tmp_file);
     gsr->len = ftell(gam->bestn_tmp_file)
              - gsr->pos;
     return gsr;
@@ -268,7 +254,8 @@ static void GAM_QueryResult_push(GAM_QueryResult *gqr,
                                           gam_result->query,
                                           gam_result->target,
                                           alignment,
-                                          gam_result->user_data);
+                                          gam_result->user_data,
+                                          gam_result->self_data);
     PQueue_push(gqr->pq, gsr);
     return;
     }
@@ -337,7 +324,7 @@ static void GAM_QueryResult_submit(GAM_QueryResult *gqr,
 /**/
 
 static gboolean GAM_QueryResult_report_traverse_func(gpointer data,
-                                                   gpointer user_data){
+                                                     gpointer user_data){
     register GAM_StoredResult *gsr = data;
     register GPtrArray *result_list = user_data;
     g_ptr_array_add(result_list, gsr);
@@ -424,14 +411,14 @@ GAM *GAM_create(Alphabet_Type query_type, Alphabet_Type target_type,
     register GAM *gam = g_new0(GAM, 1);
     register gint i;
     register C4_Span *span;
-    gam->ref_count = 1;
+    gam->thread_ref = ThreadRef_create();
     gam->dna_submat = Submat_share(dna_submat);
     gam->protein_submat = Submat_share(protein_submat);
     gam->translate = Translate_share(translate);
     gam->gas = GAM_ArgumentSet_create(NULL);
     if(use_exhaustive && gam->gas->use_subopt)
         g_warning("Exhaustively generating suboptimal alignments"
-                  " will be VERY SLOW");
+                  " will be VERY SLOW: use -S no");
     gam->query_type = query_type;
     gam->target_type = target_type;
     if(gam->gas->best_n){
@@ -446,7 +433,7 @@ GAM *GAM_create(Alphabet_Type query_type, Alphabet_Type target_type,
     gam->model = Model_Type_get_model(gam->gas->type,
                                       query_type, target_type);
     if((!use_exhaustive) && (!C4_Model_is_local(gam->model)))
-        g_error("Cannot perform heuristic alignment using non-local model");
+        g_error("Cannot perform heuristic alignments using non-local models: use -E");
     gam->match_list = GAM_build_match_list(gam->model);
     if(use_exhaustive){
         gam->optimal = Optimal_create(gam->model, NULL,
@@ -480,12 +467,15 @@ GAM *GAM_create(Alphabet_Type query_type, Alphabet_Type target_type,
         if(gam->max_target_span < span->max_target)
             gam->max_target_span = span->max_target;
         }
+#ifdef USE_PTHREADS
+    pthread_mutex_init(&gam->gam_lock, NULL);
+#endif /* USE_PTHREADS */
     return gam;
     }
 
 GAM *GAM_share(GAM *gam){
     g_assert(gam);
-    gam->ref_count++;
+    ThreadRef_share(gam->thread_ref);
     return gam;
     }
 
@@ -557,8 +547,11 @@ static void GAM_percent_threshold_tree_destroy(
 
 void GAM_destroy(GAM *gam){
     g_assert(gam);
-    if(--gam->ref_count)
+    if(ThreadRef_destroy(gam->thread_ref))
         return;
+#ifdef USE_PTHREADS
+    pthread_mutex_destroy(&gam->gam_lock);
+#endif /* USE_PTHREADS */
     g_assert(gam->model);
     g_ptr_array_free(gam->match_list, TRUE);
     C4_Model_destroy(gam->model);
@@ -639,6 +632,8 @@ static GAM_Result *GAM_Result_create(GAM *gam,
     gam_result->target = Sequence_share(target);
     gam_result->user_data = Model_Type_create_data(gam->gas->type,
                                                    query, target);
+    gam_result->self_data = Model_Type_create_data(gam->gas->type,
+                                                   query, query);
     gam_result->subopt = SubOpt_create(query->len, target->len);
     return gam_result;
     }
@@ -1108,7 +1103,8 @@ static void GAM_Result_geneseed_filter(GAM *gam, Comparison *comparison){
     /* Keep HSPs marked as keep */
     hsp_id = GAM_Result_geneseed_select(comparison->dna_hspset,
                  gsd.fwd_keep, gsd.rev_keep, hsp_id);
-    hsp_id = GAM_Result_geneseed_select(comparison->protein_hspset, gsd.fwd_keep, gsd.rev_keep, hsp_id);
+    hsp_id = GAM_Result_geneseed_select(comparison->protein_hspset,
+                 gsd.fwd_keep, gsd.rev_keep, hsp_id);
     hsp_id = GAM_Result_geneseed_select(comparison->codon_hspset,
                  gsd.fwd_keep, gsd.rev_keep, hsp_id);
     /**/
@@ -1152,8 +1148,10 @@ GAM_Result *GAM_Result_heuristic_create(GAM *gam,
         /* Raise score threshold to geneseed
          * to prevent low-scoring subopt alignments.
          */
+        GAM_lock(gam);
         if(gam->gas->threshold < has->geneseed_threshold)
             gam->gas->threshold = has->geneseed_threshold;
+        GAM_unlock(gam);
         GAM_Result_geneseed_filter(gam, comparison);
         }
     if(!Comparison_has_hsps(comparison))
@@ -1179,18 +1177,26 @@ GAM_Result *GAM_Result_exhaustive_create(GAM *gam,
                                          Sequence *target){
     register Alignment *alignment;
     register C4_Score threshold;
-    register GAM_Result *gam_result = GAM_Result_create(gam,
-                                                        query, target);
+    register GAM_Result *gam_result;
     register OPair *opair;
     g_assert(gam->optimal);
+    GAM_lock(gam);
+    Sequence_lock(query);
+    Sequence_lock(target);
+    gam_result = GAM_Result_create(gam, query, target);
+    Sequence_unlock(query);
+    Sequence_unlock(target);
     opair = OPair_create(gam->optimal, gam_result->subopt,
                          query->len, target->len, gam_result->user_data);
+    GAM_unlock(gam);
     if(gam->verbosity > 1)
         g_message("Exhaustive alignment of [%s] [%s]",
-                 query->id, target->id);
+                  query->id, target->id);
     /**/
     do {
+        GAM_lock(gam);
         threshold = GAM_get_query_threshold(gam, query);
+        GAM_unlock(gam);
         alignment = OPair_next_path(opair, threshold);
         if(!alignment)
             break;
@@ -1224,17 +1230,22 @@ void GAM_Result_destroy(GAM_Result *gam_result){
         }
     Model_Type_destroy_data(gam_result->gam->gas->type,
                             gam_result->user_data);
+    Model_Type_destroy_data(gam_result->gam->gas->type,
+                            gam_result->self_data);
     Sequence_destroy(gam_result->query);
     Sequence_destroy(gam_result->target);
+    GAM_lock(gam_result->gam);
     GAM_destroy(gam_result->gam);
     SubOpt_destroy(gam_result->subopt);
+    GAM_unlock(gam_result->gam);
     g_free(gam_result);
     return;
     }
 
 static void GAM_display_alignment(GAM *gam, Alignment *alignment,
             Sequence *query, Sequence *target,
-            gint result_id, gint rank, gpointer user_data, FILE *fp){
+            gint result_id, gint rank,
+            gpointer user_data, gpointer self_data, FILE *fp){
     if(gam->gas->show_alignment)
         Alignment_display(alignment, query, target,
                           gam->dna_submat, gam->protein_submat,
@@ -1246,15 +1257,16 @@ static void GAM_display_alignment(GAM *gam, Alignment *alignment,
     if(gam->gas->show_vulgar)
         Alignment_display_vulgar(alignment, query, target, fp);
     if(gam->gas->show_query_gff)
-        Alignment_display_gff(alignment, query, target,
-                              TRUE, FALSE, result_id, fp);
+        Alignment_display_gff(alignment, query, target, gam->translate,
+                              TRUE, FALSE, result_id, user_data, fp);
     if(gam->gas->show_target_gff)
-        Alignment_display_gff(alignment, query, target, FALSE,
-             Model_Type_has_genomic_target(gam->gas->type), result_id, fp);
+        Alignment_display_gff(alignment, query, target, gam->translate,
+             FALSE, Model_Type_has_genomic_target(gam->gas->type),
+             result_id, user_data, fp);
     if(gam->gas->ryo)
         Alignment_display_ryo(alignment, query, target,
                               gam->gas->ryo, gam->translate, rank,
-                              user_data, fp);
+                              user_data, self_data, fp);
     fflush(fp);
     return;
     }
@@ -1267,7 +1279,7 @@ static void GAM_Result_display(GAM_Result *gam_result){
         alignment = gam_result->alignment_list->pdata[i];
         GAM_display_alignment(gam_result->gam, alignment,
                 gam_result->query, gam_result->target,
-                i+1, 0, gam_result->user_data, stdout);
+                i+1, 0, gam_result->user_data, gam_result->self_data, stdout);
         }
     return;
     }
@@ -1275,6 +1287,7 @@ static void GAM_Result_display(GAM_Result *gam_result){
 void GAM_Result_submit(GAM_Result *gam_result){
     register GAM_QueryResult *gqr;
     g_assert(gam_result);
+    GAM_lock(gam_result->gam);
     if(gam_result->gam->gas->best_n){
         gqr = g_tree_lookup(gam_result->gam->bestn_tree,
                             gam_result->query->id);
@@ -1290,6 +1303,21 @@ void GAM_Result_submit(GAM_Result *gam_result){
     } else {
         GAM_Result_display(gam_result);
         }
+    GAM_unlock(gam_result->gam);
+    return;
+    }
+
+void GAM_lock(GAM *gam){
+#ifdef USE_PTHREADS
+    pthread_mutex_lock(&gam->gam_lock);
+#endif /* USE_PTHREADS */
+    return;
+    }
+
+void GAM_unlock(GAM *gam){
+#ifdef USE_PTHREADS
+    pthread_mutex_unlock(&gam->gam_lock);
+#endif /* USE_PTHREADS */
     return;
     }
 
