@@ -3,7 +3,7 @@
 *  The exonerate server                                          *
 *                                                                *
 *  Guy St.C. Slater..   mailto:guy@ebi.ac.uk                     *
-*  Copyright (C) 2000-2008.  All Rights Reserved.                *
+*  Copyright (C) 2000-2009.  All Rights Reserved.                *
 *                                                                *
 *  This source code is distributed under the terms of the        *
 *  GNU General Public License, version 3. See the file COPYING   *
@@ -66,6 +66,10 @@ static Exonerate_Server *Exonerate_Server_create(gchar *input_path,
     if(preload)
         Dataset_preload_seqs(exonerate_server->dataset);
     exonerate_server->verbosity = verbosity;
+    if(verbosity >= 1){
+        Dataset_info(exonerate_server->dataset);
+        Index_info(exonerate_server->index);
+        }
     return exonerate_server;
     }
 
@@ -93,6 +97,9 @@ typedef struct {
                gint  dna_hsp_threshold;
                gint  protein_hsp_threshold;
                gint  codon_hsp_threshold;
+               gint  dna_word_limit;
+               gint  protein_word_limit;
+               gint  codon_word_limit;
                gint  dna_hsp_dropoff;
                gint  protein_hsp_dropoff;
                gint  codon_hsp_dropoff;
@@ -118,6 +125,9 @@ static Exonerate_Server_Connection *Exonerate_Server_Connection_create(void){
     connection->dna_hsp_threshold = has->dna_hsp_threshold;
     connection->protein_hsp_threshold = has->protein_hsp_threshold;
     connection->codon_hsp_threshold = has->codon_hsp_threshold;
+    connection->dna_word_limit = has->dna_word_limit;
+    connection->protein_word_limit = has->protein_word_limit;
+    connection->codon_word_limit = has->codon_word_limit;
     connection->dna_hsp_dropoff = has->dna_hsp_dropoff;
     connection->protein_hsp_dropoff = has->protein_hsp_dropoff;
     connection->codon_hsp_dropoff = has->codon_hsp_dropoff;
@@ -143,7 +153,6 @@ static void Exonerate_Server_Connection_destroy(
 /**/
 
 static gpointer Exonerate_Server_Connection_open(gpointer user_data){
-    g_message("[%s]", __FUNCTION__);
     return Exonerate_Server_Connection_create();
     }
 
@@ -151,7 +160,6 @@ static void Exonerate_Server_Connection_close(gpointer connection_data,
                                               gpointer user_data){
     register Exonerate_Server_Connection *server_connection
         = connection_data;
-    g_message("[%s]", __FUNCTION__);
     Exonerate_Server_Connection_destroy(server_connection);
     return;
     }
@@ -221,10 +229,15 @@ static gchar *Exonerate_Server_help(void){
         "\n"
         "    valid parameters:\n"
         "        seedrepeat\n"
+        "\n"
         "        dnahspthreshold\n"
         "        proteinhspthreshold\n"
         "        codonhspthreshold\n"
         "\n"
+        "        dnawordlimit\n"
+        "        proteinwordlimit\n"
+        "        codonwordlimit\n"
+        "\n"
         "        geneseedthreshold\n"
         "        geneseedrepeat\n"
         "        maxqueryspan\n"
@@ -392,6 +405,13 @@ static Sequence *Exonerate_Server_get_query(Index *index,
         HSP_Param_set_codon_hsp_threshold(connection->hsp_param,
                                           connection->codon_hsp_threshold);
         /**/
+        HSP_Param_set_dna_word_limit(connection->hsp_param,
+                                     connection->dna_word_limit);
+        HSP_Param_set_protein_word_limit(connection->hsp_param,
+                                         connection->protein_word_limit);
+        HSP_Param_set_codon_word_limit(connection->hsp_param,
+                                       connection->codon_word_limit);
+        /**/
         HSP_Param_set_dna_hsp_dropoff(connection->hsp_param,
                                       connection->dna_hsp_dropoff);
         HSP_Param_set_protein_hsp_dropoff(connection->hsp_param,
@@ -458,6 +478,43 @@ static gchar *Exonerate_Server_set_param_codonhspthreshold(
 
 /**/
 
+static gchar *Exonerate_Server_set_param_dnawordlimit(
+              Exonerate_Server_Connection *connection, GPtrArray *word_list){
+    register gint dnawordlimit = atoi(word_list->pdata[3]);
+    if(dnawordlimit < 0)
+        return g_strdup_printf("error: dnawordlimit must be >= 0\n");
+    connection->dna_word_limit = dnawordlimit;
+    if(connection->hsp_param)
+        HSP_Param_set_dna_word_limit(connection->hsp_param, dnawordlimit);
+    return g_strdup_printf("ok: set\n");
+    }
+
+static gchar *Exonerate_Server_set_param_proteinwordlimit(
+              Exonerate_Server_Connection *connection, GPtrArray *word_list){
+    register gint proteinwordlimit = atoi(word_list->pdata[3]);
+    if(proteinwordlimit < 0)
+        return g_strdup_printf("error: proteinwordlimit must be >= 0\n");
+    connection->protein_word_limit = proteinwordlimit;
+    if(connection->hsp_param)
+        HSP_Param_set_protein_word_limit(connection->hsp_param,
+                                         proteinwordlimit);
+    return g_strdup_printf("ok: set\n");
+    }
+
+static gchar *Exonerate_Server_set_param_codonwordlimit(
+              Exonerate_Server_Connection *connection, GPtrArray *word_list){
+    register gint codonwordlimit = atoi(word_list->pdata[3]);
+    if(codonwordlimit < 0)
+        return g_strdup_printf("error: codonwordlimit must be >= 0\n");
+    connection->codon_word_limit = codonwordlimit;
+    if(connection->hsp_param)
+        HSP_Param_set_codon_word_limit(connection->hsp_param,
+                                       codonwordlimit);
+    return g_strdup_printf("ok: set\n");
+    }
+
+/**/
+
 static gchar *Exonerate_Server_set_param_dnahspdropoff(
               Exonerate_Server_Connection *connection, GPtrArray *word_list){
     register gint dnahspdropoff = atoi(word_list->pdata[3]);
@@ -545,6 +602,15 @@ static gchar *Exonerate_Server_set_param(Exonerate_Server_Connection *connection
     } else if(!strcmp(name, "codonhspthreshold")){
         reply = Exonerate_Server_set_param_codonhspthreshold(connection,
                                                              word_list);
+    } else if(!strcmp(name, "dnawordlimit")){
+        reply = Exonerate_Server_set_param_dnawordlimit(connection,
+                                                          word_list);
+    } else if(!strcmp(name, "proteinwordlimit")){
+        reply = Exonerate_Server_set_param_proteinwordlimit(connection,
+                                                              word_list);
+    } else if(!strcmp(name, "codonwordlimit")){
+        reply = Exonerate_Server_set_param_codonwordlimit(connection,
+                                                            word_list);
     } else if(!strcmp(name, "dnahspdropoff")){
         reply = Exonerate_Server_set_param_dnahspdropoff(connection, word_list);
     } else if(!strcmp(name, "proteinhspdropoff")){
@@ -600,7 +666,10 @@ static gboolean Exonerate_Server_process(gchar *msg, gchar **reply,
             (*reply) = NULL;
             keep_connection = FALSE;
         } else if(!strcmp(word, "dbinfo")){
-            (*reply) = g_strdup_printf("dbinfo: %s %s %lld %lld %lld\n",
+            (*reply) = g_strdup_printf("dbinfo: %s %s"
+                   " %" CUSTOM_GUINT64_FORMAT
+                   " %" CUSTOM_GUINT64_FORMAT
+                   " %" CUSTOM_GUINT64_FORMAT "\n",
                   (server->dataset->header->type & 1)?"dna":"protein",
                   (server->dataset->header->type & (1<<1))
                       ?"softmasked":"unmasked",
