@@ -3,7 +3,7 @@
 *  C4 dynamic programming library - Seeded Dynamic Programming   *
 *                                                                *
 *  Guy St.C. Slater..   mailto:guy@ebi.ac.uk                     *
-*  Copyright (C) 2000-2008.  All Rights Reserved.                *
+*  Copyright (C) 2000-2009.  All Rights Reserved.                *
 *                                                                *
 *  This source code is distributed under the terms of the        *
 *  GNU General Public License, version 3. See the file COPYING   *
@@ -300,7 +300,7 @@ SDP *SDP_create(C4_Model *model){
     register SDP *sdp = g_new0(SDP, 1);
     register C4_Portal *portal;
     /**/
-    sdp->ref_count = 1;
+    sdp->thread_ref = ThreadRef_create();
     sdp->sas = SDP_ArgumentSet_create(NULL);
     g_assert(model);
     g_assert(!model->is_open);
@@ -356,13 +356,13 @@ SDP *SDP_create(C4_Model *model){
 
 SDP *SDP_share(SDP *sdp){
     g_assert(sdp);
-    sdp->ref_count++;
+    ThreadRef_share(sdp->thread_ref);
     return sdp;
     }
 
 void SDP_destroy(SDP *sdp){
     g_assert(sdp);
-    if(--sdp->ref_count)
+    if(ThreadRef_destroy(sdp->thread_ref))
         return;
     if(sdp->find_starts_scheduler)
         Scheduler_destroy(sdp->find_starts_scheduler);
