@@ -3,7 +3,7 @@
 *  C4 dynamic programming library - Seeded Dynamic Programming   *
 *                                                                *
 *  Guy St.C. Slater..   mailto:guy@ebi.ac.uk                     *
-*  Copyright (C) 2000-2008.  All Rights Reserved.                *
+*  Copyright (C) 2000-2009.  All Rights Reserved.                *
 *                                                                *
 *  This source code is distributed under the terms of the        *
 *  GNU General Public License, version 3. See the file COPYING   *
@@ -33,6 +33,7 @@ extern "C" {
 #include "sparsecache.h"
 #include "boundary.h"
 #include "scheduler.h"
+#include "threadref.h"
 
 /**/
 
@@ -66,7 +67,7 @@ typedef struct SDP_Seed {
 /**/
 
 typedef struct {
-               gint  ref_count;
+          ThreadRef *thread_ref;
     SDP_ArgumentSet *sas;
            C4_Model *model;
            gboolean  use_boundary;
