@@ -3,7 +3,7 @@
 *  Simple Alphabet Object                                        *
 *                                                                *
 *  Guy St.C. Slater..   mailto:guy@ebi.ac.uk                     *
-*  Copyright (C) 2000-2008.  All Rights Reserved.                *
+*  Copyright (C) 2000-2009.  All Rights Reserved.                *
 *                                                                *
 *  This source code is distributed under the terms of the        *
 *  GNU General Public License, version 3. See the file COPYING   *
@@ -49,6 +49,7 @@ gchar *Alphabet_Argument_parse_alphabet_type(gchar *arg_string, gpointer data);
 
 typedef struct {
              gint  ref_count;
+         gboolean  is_soft_masked;
            guchar *member;
     Alphabet_Type  type;
            guchar *is_member;
@@ -113,6 +114,12 @@ gchar *Alphabet_aa2tla(gchar aa);
 /* Give three-letter-abbreviation for amino acid.
  * eg 'M' -> "Met" etc */
 
+gchar *Alphabet_nt2ambig(gchar nt);
+/* Returns the ambiguity bases for the given nucleotide
+ * The returned string does not need to be freed.
+ */
+
+
 /**/
 
 #ifdef __cplusplus
