@@ -3,7 +3,7 @@
 *  Nucleotide Translation Code                                   *
 *                                                                *
 *  Guy St.C. Slater..   mailto:guy@ebi.ac.uk                     *
-*  Copyright (C) 2000-2008.  All Rights Reserved.                *
+*  Copyright (C) 2000-2009.  All Rights Reserved.                *
 *                                                                *
 *  This source code is distributed under the terms of the        *
 *  GNU General Public License, version 3. See the file COPYING   *
@@ -24,8 +24,8 @@ static void reverse_func(gchar *dna, gint length, gpointer user_data){
 
 static void check_translate(Translate *t, gchar *codon, gchar crib){
     register gchar aa = Translate_codon(t, codon);
-    g_assert(aa == crib);
     g_message("Translate [%s] -> [%c] (crib:[%c]", codon, aa, crib);
+    g_assert(aa == crib);
     return;
     }
 
@@ -39,10 +39,14 @@ static void check_reverse_translate(Translate *t, gchar *word, gint crib){
     }
 
 gint Argument_main(Argument *arg){
-    register Translate *t = Translate_create(FALSE);
+    register Translate *t;
+    Translate_ArgumentSet_create(arg);
+    Argument_process(arg, "translate.test", NULL, NULL);
+    t = Translate_create(FALSE);
     check_translate(t, "ATG", 'M');
     check_translate(t, "GGX", 'G');
     check_translate(t, "NGT", 'X');
+    check_translate(t, "TGA", '*');
     /**/
     check_reverse_translate(t, "S*MXG",   72);
     check_reverse_translate(t, "SSSSS", 7776);
