@@ -3,7 +3,7 @@
 *  A simple bitarray data structure                              *
 *                                                                *
 *  Guy St.C. Slater..   mailto:guy@ebi.ac.uk                     *
-*  Copyright (C) 2000-2008.  All Rights Reserved.                *
+*  Copyright (C) 2000-2009.  All Rights Reserved.                *
 *                                                                *
 *  This source code is distributed under the terms of the        *
 *  GNU General Public License, version 3. See the file COPYING   *
@@ -52,6 +52,8 @@ gboolean  BitArray_get_bit(BitArray *ba, guint64 pos);
 
     void  BitArray_write_int(guint64 num, FILE *fp);
  guint64  BitArray_read_int(FILE *fp);
+    void  BitArray_int_print(guint64 num);
+   gchar *BitArray_int_sprintf(guint64 num);
 
 #ifdef __cplusplus
 }
