@@ -3,7 +3,7 @@
 *  Deja-vu library for fast linear space repeat finding          *
 *                                                                *
 *  Guy St.C. Slater..   mailto:guy@ebi.ac.uk                     *
-*  Copyright (C) 2000-2008.  All Rights Reserved.                *
+*  Copyright (C) 2000-2009.  All Rights Reserved.                *
 *                                                                *
 *  This source code is distributed under the terms of the        *
 *  GNU General Public License, version 3. See the file COPYING   *
@@ -69,9 +69,8 @@ static gint DejaVu_init(DejaVu *dv, guchar *filter){
             continue;
         if(first[ch] == -1)
             first[ch] = i;
-        if(last[ch] != -1){
+        if(last[ch] != -1)
             dv->next[last[ch]] = i;
-            }
         last[ch] = i;
         }
     for(i = 0; i < ALPHABET_SIZE; i++){
