@@ -3,7 +3,7 @@
 *  VFSM Library : complete trie navigation                       *
 *                                                                *
 *  Guy St.C. Slater..   mailto:guy@ebi.ac.uk                     *
-*  Copyright (C) 2000-2008.  All Rights Reserved.                *
+*  Copyright (C) 2000-2009.  All Rights Reserved.                *
 *                                                                *
 *  This source code is distributed under the terms of the        *
 *  GNU General Public License, version 3. See the file COPYING   *
@@ -22,7 +22,11 @@
 #ifdef G_HAVE_GINT64
 typedef guint64 VFSM_Int;
 #define VFSM_MAX G_GINT64_CONSTANT(18446744073709551615U)
+#ifdef CUSTOM_GUINT64_FORMAT
+#define VFSM_PRINT_FORMAT CUSTOM_GUINT64_FORMAT
+#else /* CUSTOM_GUINT64_FORMAT */
 #define VFSM_PRINT_FORMAT "llu"
+#endif /* CUSTOM_GUINT64_FORMAT */
 #else /* G_HAVE_GINT64 */
 typedef guint32 VFSM_Int;
 #define VFSM_MAX 4294967295U
