@@ -3,7 +3,7 @@
 *  fastafetch : fetch a sequence from a fasta format file        *
 *                                                                *
 *  Guy St.C. Slater..   mailto:guy@ebi.ac.uk                     *
-*  Copyright (C) 2000-2008.  All Rights Reserved.                *
+*  Copyright (C) 2000-2009.  All Rights Reserved.                *
 *                                                                *
 *  This source code is distributed under the terms of the        *
 *  GNU General Public License, version 3. See the file COPYING   *
@@ -44,7 +44,7 @@ static gint fasta_index_read_to_char(FILE *fp, gchar c, GString *str){
 static CompoundFile_Pos fasta_index_lookup(FastaDB *fdb, FILE *index_fp,
                                            gchar *id){
     register long left, right, pos;
-    CompoundFile_Pos result = -1;
+    CompoundFile_Pos_scan_type result = -1;
     register GString *curr = g_string_sized_new(100);
     register gint cond;
     left = 0;
@@ -80,7 +80,7 @@ static CompoundFile_Pos fasta_index_lookup(FastaDB *fdb, FILE *index_fp,
             }
         }
     g_string_free(curr, TRUE);
-    return result;
+    return (CompoundFile_Pos)result;
     }
 
 static FastaDB_Seq *fasta_index_fetch(FastaDB *fdb, FILE *index_fp,
