@@ -3,7 +3,7 @@
 *  fastatranslate: a utility to translate fasta format sequences *
 *                                                                *
 *  Guy St.C. Slater..   mailto:guy@ebi.ac.uk                     *
-*  Copyright (C) 2000-2008.  All Rights Reserved.                *
+*  Copyright (C) 2000-2009.  All Rights Reserved.                *
 *                                                                *
 *  This source code is distributed under the terms of the        *
 *  GNU General Public License, version 3. See the file COPYING   *
@@ -37,7 +37,8 @@ static void fasta_translate_seq(FastaDB_Seq *fdbs,
     } else {
         aa_seq = Sequence_translate(fdbs->seq, translate, frame);
         }
-    Sequence_print_fasta(aa_seq, stdout, FALSE);
+    if(aa_seq->len)
+        Sequence_print_fasta(aa_seq, stdout, FALSE);
     Sequence_destroy(aa_seq);
     return;
     }
@@ -47,7 +48,7 @@ int Argument_main(Argument *arg){
     register FastaDB_Seq *fdbs;
     register ArgumentSet *as
            = ArgumentSet_create("Sequence Input Options");
-    register Translate *translate = Translate_create(FALSE);
+    register Translate *translate;
     register Alphabet *dna_alphabet
            = Alphabet_create(Alphabet_Type_DNA, FALSE);
     register Alphabet *protein_alphabet
@@ -66,6 +67,7 @@ int Argument_main(Argument *arg){
         "A utility to translate fasta format sequences\n"
         "Guy St.C. Slater. guy@ebi.ac.uk. 2000-2003.\n", NULL);
     fdb = FastaDB_open(query_path, dna_alphabet);
+    translate = Translate_create(FALSE);
     while((fdbs = FastaDB_next(fdb, FastaDB_Mask_ALL))){
         fasta_translate_seq(fdbs, translate, frame, protein_alphabet);
         FastaDB_Seq_destroy(fdbs);
